/*
 * $Id: IDDLauncher.java,v 1.15 2005/08/11 13:41:31 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 14.06.2005 08:41:52 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.client;

import org.deltadebugging.ddstate.comparator.IGraphComparator;
import org.deltadebugging.ddstate.comparator.exceptions.GraphComparatorException;
import org.deltadebugging.ddstate.launcher.client.exception.DeltaComputationException;
import org.deltadebugging.ddstate.launcher.ddrun.IDDLaunchDescription;
import org.deltadebugging.ddstate.launcher.ddrun.IDDRunDescription;
import org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult;
import org.deltadebugging.ddstate.launcher.matcher.IMatchPointFinder;
import org.deltadebugging.ddstate.launcher.tester.exception.TesterException;
import org.eclipse.core.runtime.IProgressMonitor;

/**
 * COMMENTME Comment type IDDLauncher
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface IDDLauncher {

	public void start(IDDLaunchDescription ddLaunchDesc,
			IProgressMonitor monitor);

	/**
	 * Returns the launch description.
	 * 
	 * @return the launch description.
	 */
	public IDDLaunchDescription getLaunchDescription();

	/**
	 * Set the launch description.
	 * 
	 * @param launchDesc
	 *            launch description.
	 */
	public void setLaunchDescription(IDDLaunchDescription launchDesc);

	/**
	 * Returns the test runner for the passing run.
	 * 
	 * @return the test runner for the passing run.
	 */
	public IExtTestRunner getPassTestRunner();

	/**
	 * Set the test runner for the passing run.
	 * 
	 * @param passTestRunner
	 *            test runner for the passing run.
	 */
	public void setPassTestRunner(IExtTestRunner passTestRunner);

	/**
	 * Returns the test runner for the failing run.
	 * 
	 * @return the test runner for the failing run.
	 */
	public IExtTestRunner getFailTestRunner();

	/**
	 * Set the test runner for the failing run.
	 * 
	 * @param failTestRunner
	 *            test runner for the failing run.
	 */
	public void setFailTestRunner(IExtTestRunner failTestRunner);

	/**
	 * Define the match point finder that is used to find comparable locations
	 * within the passing and the failing run. At this computed locations we
	 * apply delta debugging.
	 * 
	 * @param matchPointFinder
	 */
	public void setMatchPointFinder(IMatchPointFinder matchPointFinder);

	/**
	 * Returns the match point finder, that should be used to find all
	 * comparable locations within the passing and the failing run. Maybe, if no
	 * match point finder is set this function returns <code>null</code>.
	 * 
	 * @return the match point finder set to find all comparable locations
	 *         within the passing and the failing run.
	 */
	public IMatchPointFinder getMatchPointFinder();

	/**
	 * Enable the match point finder.
	 */
	public void enableMatchPointFinder();

	/**
	 * Disable the match point finder. This includes that in the dd run
	 * description the match points are already set.
	 */
	public void disableMatchPointFinder();

	/**
	 * Returns true if the match point finder is enabled, otherwise false.
	 * 
	 * @return true if the match point finder is enbaled, otherwise false.
	 */
	public boolean isMatchPointFinderEnabled();

	/**
	 * Define the graph comparator that is used to compare two memory graph and
	 * compute the deltas set of this two memory graphs.
	 * 
	 * @param graphComparator
	 */
	public void setGraphComparator(IGraphComparator graphComparator);

	/**
	 * Returns the graph comparator used to compare the memory graphs of the
	 * passing and the failing run.
	 * 
	 * @return the graph comparator used to compare the memory graphs of the
	 *         passing and the failing run.
	 */
	public IGraphComparator getGraphComparator();

	public IDDRunResult computeRelevantDeltas(IDDRunDescription ddRunDesc,
			IProgressMonitor monitor) throws GraphComparatorException,
			DeltaComputationException, TestRunnerException, TesterException;
}
