/*
 * $Id: ITestDescription.java,v 1.7 2005/08/15 12:25:15 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 14.02.2005 14:08:19 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.client;

import org.deltadebugging.ddstate.launcher.matcher.breakpoint.IBreakpointLocation;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.launching.IVMInstall;

/**
 * The test description store all usefull informations to run either the passing
 * or the failing junit test. It also includes the information where the run
 * should supsended.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface ITestDescription {

	/**
	 * Returns the java project which contains the test class.
	 * 
	 * @return the java project which contains the test class.
	 */
	public IJavaProject getJavaProject();

	/**
	 * Returns the vm installation used to execute the test run.
	 * 
	 * @return the vm installation used to execute the test run.
	 */
	public IVMInstall getVMInstall();

	/**
	 * Returns the breakpoint location, where the test run should suspended.
	 * 
	 * @return the breakpoint location, where the test run should suspended.
	 */
	public IBreakpointLocation getBreakpointLocation();

	/**
	 * Returns the junit test description including the test class and test
	 * method name.
	 * 
	 * @return the junit test description including the test class and test
	 *         method name.
	 */
	public IJUnitTestDescription getJUnitTestDescription();
}