/*
 * $Id: TestDescription.java,v 1.8 2005/08/15 12:25:15 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 12.02.2005 12:35:16 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.client;

import org.deltadebugging.ddstate.launcher.matcher.breakpoint.IBreakpointLocation;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.launching.IVMInstall;

/**
 * COMMENTME Comment type TestDescription
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class TestDescription implements ITestDescription {

	private final IJavaProject fJavaProject;

	private final IVMInstall fVMInstall;

	private final IJUnitTestDescription fJUnitTestDesc;

	private final IBreakpointLocation fBreakpointLocation;

	public TestDescription(IJavaProject javaProject, IVMInstall vmInstall,
			IBreakpointLocation breakpointDesc,
			IJUnitTestDescription junitTestDesc) {
		this.fJavaProject = javaProject;
		this.fVMInstall = vmInstall;

		this.fBreakpointLocation = breakpointDesc;

		this.fJUnitTestDesc = junitTestDesc;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.client.ITestDescription#getJavaProject()
	 */
	public IJavaProject getJavaProject() {
		return this.fJavaProject;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.client.ITestDescription#getVMInstall()
	 */
	public IVMInstall getVMInstall() {
		return this.fVMInstall;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.client.ITestDescription#getBreakpointLocation()
	 */
	public IBreakpointLocation getBreakpointLocation() {
		return this.fBreakpointLocation;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.client.ITestDescription#getJUnitTestDescription()
	 */
	public IJUnitTestDescription getJUnitTestDescription() {
		return this.fJUnitTestDesc;
	}

	public String toString() {
		StringBuffer result = new StringBuffer("TestDescription:\n");

		result.append("\tJavaProject: " + this.fJavaProject.getElementName()
				+ "\n");
		result.append("\tJRE        : " + this.fVMInstall.getName() + "\n");
		result.append("\tJUnitTest  :\n");
		result.append("\t\tTestClassName = "
				+ this.fJUnitTestDesc.getTestCaseClassName() + "\n");
		result.append("\t\tTestName      = "
				+ this.fJUnitTestDesc.getTestName() + "\n");
		result.append(this.fBreakpointLocation.toString());

		return result.toString();
	}
}