/*
 * $Id: DDRunDescription.java,v 1.9 2005/08/15 12:31:22 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 28.07.2005 17:51:24 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.ddrun;

import org.deltadebugging.ddstate.launcher.client.ITestDescription;
import org.deltadebugging.ddstate.launcher.client.TestDescription;
import org.deltadebugging.ddstate.launcher.matcher.IMatchPoint;
import org.deltadebugging.ddstate.launcher.matcher.breakpoint.IBreakpointLocation;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.launching.IVMInstall;

/**
 * The delta debugging run description includes all information to excecute one
 * delta debugging run on well defined locations (match point) in the test runs.
 * This description is used to compute all relevant deltas between the failing
 * and the passing run at the given match point.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class DDRunDescription implements IDDRunDescription {

	private int fDDRunID = -1;

	private final ITestDescription fPassTestDescription;

	private final ITestDescription fFailTestDescription;

	public DDRunDescription(ITestDescription passTestDescription,
			ITestDescription failTestDescription) {
		this.fPassTestDescription = passTestDescription;
		this.fFailTestDescription = failTestDescription;
	}

	public DDRunDescription(IDDLaunchDescription launchDesc,
			IMatchPoint matchPoint) {
		this(launchDesc, matchPoint.getPassingBreakpointLocation(), matchPoint
				.getFailingBreakpointLocation());
	}

	public DDRunDescription(IDDLaunchDescription launchDesc,
			IBreakpointLocation passBreakpoint, IBreakpointLocation failBreakpoint) {

		IJavaProject javaProject = launchDesc.getJavaProject();
		IVMInstall vmInstall = launchDesc.getVMInstall();

		this.fPassTestDescription = new TestDescription(javaProject, vmInstall,
				passBreakpoint, launchDesc.getPassJUnitTestDescription());

		this.fFailTestDescription = new TestDescription(javaProject, vmInstall,
				failBreakpoint, launchDesc.getFailJUnitTestDescription());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunDescription#setDDRunID(int)
	 */
	public void setDDRunID(int ddRunID) {
		this.fDDRunID = ddRunID;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunDescription#getDDRunID()
	 */
	public int getDDRunID() {
		return this.fDDRunID;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunDescription#getPassTestDescription()
	 */
	public ITestDescription getPassTestDescription() {
		return this.fPassTestDescription;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunDescription#getFailTestDescription()
	 */
	public ITestDescription getFailTestDescription() {
		return this.fFailTestDescription;
	}

}
