/*
 * $Id: IDDLaunchDescription.java,v 1.4 2005/08/11 08:15:52 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 28.07.2005 16:41:56 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.ddrun;

import org.deltadebugging.ddstate.launcher.client.IJUnitTestDescription;
import org.deltadebugging.ddstate.launcher.matcher.IMatchPoint;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.launching.IVMInstall;

/**
 * The launch description includes all necessary informations to launch a delta
 * debugging run.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface IDDLaunchDescription {

	public int getDDLaunchID();

	public void setDDLaunchID(int launchID);

	/**
	 * Returns the java project wich contains the junit test class for the delta
	 * debugging launch.
	 * 
	 * @return the java project used to execute the delta debugging launch.
	 */
	public IJavaProject getJavaProject();

	/**
	 * Returns the vm installation used to start the delta debugging launch.
	 * 
	 * @return the vm installation used to start the delta debugging launch.
	 */
	public IVMInstall getVMInstall();

	/**
	 * Returns all match points given to this launch description or
	 * <code>null</code> if no match points are available.
	 * 
	 * @return all match points given to this launch description or
	 *         <code>null</code> if no match points are available.
	 */
	public IMatchPoint[] getMatchPoints();

	/**
	 * Returns the test description for the passing junit test run.
	 * 
	 * @return the test description for the passing junit test run.
	 */
	public IJUnitTestDescription getPassJUnitTestDescription();

	/**
	 * Returns the test description for the failing junit test run.
	 * 
	 * @return the test description for the failing junit test run.
	 */
	public IJUnitTestDescription getFailJUnitTestDescription();

}
