/*
 * $Id: IDDRunListenerManager.java,v 1.4 2005/08/17 16:08:31 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 10.04.2005 18:05:54 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.ddrun;

import org.deltadebugging.ddcore.DeltaSet;
import org.deltadebugging.ddstate.memorygraph.IRootNode;

/**
 * This manager notify all registered listener about changes in the delta
 * debugging run. At this listener you can add and remove your dd run listener.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface IDDRunListenerManager {

	/**
	 * Registers the given listener for dd run notifications. Has no effect if
	 * an identical listener is already registered.
	 * 
	 * @param listener
	 *            listener to register
	 */
	public void addDDRunListener(IDDRunListener listener);

	/**
	 * Deregisters the given listener for dd run notifications. Has no effect if
	 * an identical listener is not already registered.
	 * 
	 * @param listener
	 *            listener to deregister
	 */
	public void removeDDRunListener(IDDRunListener listener);

	/**
	 * Returns a collection of dd run listeners registered with the dd run
	 * listener manager.
	 * 
	 * @return a collection of dd run listeners registered with the dd run
	 *         listener manager
	 */
	public IDDRunListener[] getDDRunListeners();

	/**
	 * Notify the listeners that the memory graphs was compared with each other.
	 * 
	 * @param ddRunID
	 *            the unique id of the delta debugging run.
	 * @param passGraph
	 *            the compared passing graph.
	 * @param failGraph
	 *            the compared failing graph.
	 * @param deltas
	 *            the deltas computed between passing and failing graph.
	 */
	public void notifyMemoryGraphsCompared(int ddRunID, IRootNode passGraph,
			IRootNode failGraph, DeltaSet deltas);

	/**
	 * Notify the listeners that a new delta debugging run was started. For a
	 * specific description of the dd run look at the given parameter object.
	 * 
	 * @param desc
	 *            the description with all informations to run delta debugging.
	 */
	public void notifyDDRunStarted(IDDRunDescription desc);

	/**
	 * Notify the listeners that the memory graph of the passing run and the
	 * memory graph of the failing run are extracted.
	 * 
	 * @param ddRunID
	 *            the unique id of the delta debugging run.
	 * @param passGraph
	 *            the passing graph, which was extracted.
	 * @param failGraph
	 *            the failing graph, which was extracted.
	 */
	public void notifyMemoryGraphsExtracted(int ddRunID, IRootNode passGraph,
			IRootNode failGraph);

	/**
	 * Notify the listeners that the delta debugging run has terminated.
	 * 
	 * @param result
	 *            the result of the delta debugging run.
	 */
	public void notifyDDRunTerminated(IDDRunResult result);

}