/*
 * $Id: CreateInstanceInstrumenter.java,v 1.2 2005/10/05 16:39:47 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 03.05.2005 10:00:00 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.server.instrumenter;

import org.apache.bcel.Constants;
import org.apache.bcel.classfile.JavaClass;
import org.apache.bcel.classfile.Method;
import org.apache.bcel.generic.BranchInstruction;
import org.apache.bcel.generic.ClassGen;
import org.apache.bcel.generic.ConstantPoolGen;
import org.apache.bcel.generic.InstructionFactory;
import org.apache.bcel.generic.InstructionHandle;
import org.apache.bcel.generic.InstructionList;
import org.apache.bcel.generic.MethodGen;
import org.apache.bcel.generic.ObjectType;
import org.apache.bcel.generic.Type;
import org.deltadebugging.ddstate.launcher.bcel.instrumenter.AbstractClassInstrumenter;
import org.deltadebugging.ddstate.launcher.server.instrumenter.rtclass.ClassController;

/**
 * @author lehmann
 * 
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Style - Code Templates
 */
public class CreateInstanceInstrumenter extends AbstractClassInstrumenter
        implements Constants {

    private final static String CLASSCONTROLLER_NAME = ClassController.class
            .getName();
    private final static String CLASSCONTROLLER_PATH = CLASSCONTROLLER_NAME
            .replace('.', '/');
    private final static String[] IGNORED_CLASSES_FOR_STATIC_MOD = new String[] {
            "java." };

    public CreateInstanceInstrumenter() {
        this(new String[0]);
    }

    public final static String[] DEFAULT_IGNORED_PACKAGES = new String[] {
            Object.class.getName(), CLASSCONTROLLER_NAME };

    public CreateInstanceInstrumenter(String[] ignoredPackages) {
        super(ignoredPackages);
        this.addPackagesToIgnore(DEFAULT_IGNORED_PACKAGES);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.bcel.instrumenter.AbstractClassInstrumenter#modifyClass(org.apache.bcel.classfile.JavaClass,
     *      org.apache.bcel.generic.ConstantPoolGen,
     *      org.apache.bcel.generic.ClassGen,
     *      org.apache.bcel.generic.InstructionFactory)
     */
    public JavaClass modifyClass(JavaClass clazz,
            ConstantPoolGen constantPoolGenerator, ClassGen classGenerator,
            InstructionFactory factory) {
        boolean hasEmptyConstructor = false;

        int methodCount = clazz.getMethods().length;

        if (methodCount > 0) {
            if (clazz.getMethods()[clazz.getMethods().length - 1]
                    .getSignature().equals("(L" + CLASSCONTROLLER_PATH + ";)V")) {
                hasEmptyConstructor = true;
            }
        }

        if (!hasEmptyConstructor) {
            clazz = addNewConstructor(clazz, classGenerator, factory);

            if (!this.ignoreStaticModification(clazz)) {
                clazz = suroundStaticBlockWithIfStatement(clazz, classGenerator
                        .getConstantPool(), classGenerator, factory);
            } else {
                System.out.println("Ignore Static modification for "
                        + clazz.getClassName());
            }
        }

        return clazz;
    }

    private boolean ignoreStaticModification(JavaClass clazz) {
        boolean result = false;

        for (int i = 0; i < IGNORED_CLASSES_FOR_STATIC_MOD.length
                && result == false; i++) {
            String pattern = IGNORED_CLASSES_FOR_STATIC_MOD[i];

            if (clazz.getClassName().startsWith(pattern)) {
                result = true;
            }
        }

        return result;
    }

    private JavaClass addNewConstructor(JavaClass clazz,
            ClassGen classGenerator, InstructionFactory factory) {
        MethodGen methodGenerator;
        InstructionList instructions;

        instructions = new InstructionList();

        methodGenerator = new MethodGen(ACC_PUBLIC, Type.VOID,
                new Type[] { new ObjectType(CLASSCONTROLLER_NAME) },
                new String[] { "marker" }, "<init>", clazz.getClassName(),
                instructions, classGenerator.getConstantPool());

        instructions.append(InstructionFactory.createLoad(Type.OBJECT, 0));

        if (checkIfClassIsInIgnoredPackages(clazz.getSuperclassName())) {
            instructions.append(factory.createInvoke(clazz.getSuperclassName(),
                    "<init>", Type.VOID, Type.NO_ARGS, INVOKESPECIAL));

        } else {
            instructions.append(InstructionFactory.createLoad(Type.OBJECT, 1));
            instructions.append(factory.createInvoke(clazz.getSuperclassName(),
                    "<init>", Type.VOID, new Type[] { new ObjectType(
                            CLASSCONTROLLER_NAME) }, INVOKESPECIAL));
        }
        instructions.append(InstructionFactory.createReturn(Type.VOID));

        methodGenerator.setMaxStack();
        methodGenerator.setMaxLocals();
        classGenerator.addMethod(methodGenerator.getMethod());

        clazz = classGenerator.getJavaClass();

        return clazz;
    }

    private Method getStaticMethodBlock(ClassGen classGenerator) {
        Method result = null;

        printAllMethods(classGenerator);

        Method[] methods = classGenerator.getMethods();
        for (int i = 0; i < methods.length; i++) {
            Method method = methods[i];
            if (method.getName().equals("<clinit>")) {
                result = method;
                break;
            }
        }

        return result;
    }

    private JavaClass suroundStaticBlockWithIfStatement(JavaClass clazz,
            ConstantPoolGen constantPoolGenerator, ClassGen classGenerator,
            InstructionFactory factory) {
        MethodGen methodGenerator;
        InstructionList instructions;
        InstructionList newInstructions;

        InstructionHandle lastInst;

        Method clinitMethod = this.getStaticMethodBlock(classGenerator);
        if (clinitMethod != null) {
            methodGenerator = new MethodGen(clinitMethod, clazz.getClassName(),
                    constantPoolGenerator);

            instructions = methodGenerator.getInstructionList();
            lastInst = instructions.getEnd();

            newInstructions = new InstructionList();

            newInstructions.append(factory.createFieldAccess(
                    CLASSCONTROLLER_NAME, "WITH_STATIC_BLOCK", Type.BOOLEAN,
                    GETSTATIC));

            BranchInstruction ifeq = InstructionFactory
                    .createBranchInstruction(IFEQ, null);
            ifeq.setTarget(lastInst);
            newInstructions.append(ifeq);

            instructions.insert(newInstructions);

            methodGenerator.setMaxStack();
            methodGenerator.setMaxLocals();
            classGenerator.replaceMethod(clinitMethod, methodGenerator
                    .getMethod());

            clazz = classGenerator.getJavaClass();
        }

        return clazz;
    }

}