/*
 * $Id: MatchpointTab.java,v 1.6 2006/01/31 18:57:02 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 13.12.2004 10:15:55 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.ui.tabgroup;

import org.deltadebugging.ddstate.launcher.IDDLaunchConstants;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.ui.AbstractLaunchConfigurationTab;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.search.IJavaSearchConstants;
import org.eclipse.jdt.core.search.SearchEngine;
import org.eclipse.jdt.internal.ui.dialogs.OpenTypeSelectionDialog2;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

/**
 * COMMENTME Comment type MatchpointTab
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class MatchpointTab extends AbstractLaunchConfigurationTab implements
        IDDLaunchConstants {

    private static final String ERR_MSG_FAIL_BP_CLASS_NAME = "You have to specify a valid class name for the breakpoint in the failing run.";
    private static final String ERR_MSG_PASS_BP_CLASS_NAME = "You have to specify a valid class name for the breakpoint in the passing run.";
    private static final String ERR_MSG_FAIL_BP_LINE_NUMBER = "You have to specify a valid line number for the breakpoint in the failing run (> 0).";
    private static final String ERR_MSG_PASS_BP_LINE_NUMBER = "You have to specify a valid line number for the breakpoint in the passing run (> 0).";
    private static final String ERR_MSG_FAIL_BP_HIT_COUNT = "You have to specify a valid hit count for the breakpoint in the failing run (> 0).";
    private static final String ERR_MSG_PASS_BP_HIT_COUNT = "You have to specify a valid hit count for the breakpoint in the passing run (> 0).";

    private static final String EMPTY_STRING = "";

    private Button butEnableMatchPointFinder;

    Text txtPassBPClassName;
    Text txtFailBPClassName;

    private Text txtPassBPLineNumber;
    private Text txtFailBPLineNumber;

    private Text txtPassBPHitCount;
    private Text txtFailBPHitCount;

    private Button butPassSearch;
    private Button butFailSearch;

    private Group grpFailRun;
    private Group grpPassRun;

    /**
     * @see org.eclipse.debug.ui.ILaunchConfigurationTab#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent) {
        Font font = parent.getFont();
        Composite comp = new Composite(parent, parent.getStyle());
        GridLayout layout = new GridLayout(1, false);
        comp.setLayout(layout);
        comp.setFont(font);

        comp.setLayoutData(new GridData(GridData.FILL_BOTH));
        setControl(comp);

        // empy line
        new Label(comp, SWT.NONE);
        this.createMatchPointFinderSection(comp);

        // empy line
        new Label(comp, SWT.NONE);
        this.createPassBPGroup(comp);

        // empy line
        new Label(comp, SWT.NONE);
        this.createFailBPGroup(comp);
    }

    private void createMatchPointFinderSection(Composite parent) {
        this.butEnableMatchPointFinder = new Button(parent, SWT.CHECK);
        this.butEnableMatchPointFinder
                .setText("Automatic computation of the matching points"); //$NON-NLS-1$
        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 2;
        this.butEnableMatchPointFinder.setLayoutData(gd);
        this.butEnableMatchPointFinder
                .addSelectionListener(new SelectionAdapter() {
                    public void widgetSelected(SelectionEvent e) {
                        updateLaunchConfigurationDialog();
                        setFormEnabled();                        
                    }
                });

        this.addLabel(parent, "", 0);
    }

    private void createFailBPGroup(Composite comp) {

        //
        // failing breakpoint configuration
        //
        this.grpFailRun = new Group(comp, SWT.NONE);
        this.grpFailRun.setText("Breakpoint in the failing run");
        this.grpFailRun.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        GridLayout layout = new GridLayout(2, false);
        this.grpFailRun.setLayout(layout);

        this.createFailBPSection(this.grpFailRun);
    }

    /**
     * @param parent
     */
    private void createFailBPSection(Composite parent) {
        // class name
        this.addLabel(parent, "Class Name:", 5);

        Composite compBPClassName = new Composite(parent, SWT.NONE);
        GridLayout gl = new GridLayout(2, false);
        gl.marginHeight = gl.marginWidth = 0;
        compBPClassName.setLayout(gl);
        compBPClassName.setFont(parent.getFont());
        compBPClassName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        this.txtFailBPClassName = new Text(compBPClassName, SWT.SINGLE
                | SWT.BORDER);
        this.txtFailBPClassName.setLayoutData(new GridData(
                GridData.FILL_HORIZONTAL));
        this.txtFailBPClassName.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                updateLaunchConfigurationDialog();
            }
        });

        this.butFailSearch = new Button(compBPClassName, SWT.PUSH);
        this.butFailSearch.setText("Search..."); //$NON-NLS-1$
        this.butFailSearch.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent evt) {
                String typeName = handleSearchButtonSelected();
                if (typeName != null) {
                    MatchpointTab.this.txtFailBPClassName.setText(typeName);
                }
            }
        });

        // line number
        this.addLabel(parent, "Line Number:", 5);

        this.txtFailBPLineNumber = new Text(parent, SWT.SINGLE | SWT.BORDER);
        this.txtFailBPLineNumber.setLayoutData(new GridData(
                GridData.FILL_HORIZONTAL));
        this.txtFailBPLineNumber.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent evt) {
                updateLaunchConfigurationDialog();
            }
        });

        // hit count
        this.addLabel(parent, "Hit Count:", 5);

        this.txtFailBPHitCount = new Text(parent, SWT.SINGLE | SWT.BORDER);
        this.txtFailBPHitCount.setLayoutData(new GridData(
                GridData.FILL_HORIZONTAL));
        this.txtFailBPHitCount.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent evt) {
                updateLaunchConfigurationDialog();
            }
        });
    }

    private void createPassBPGroup(Composite parent) {
        GridLayout layout;
        //
        // passing breakpoint configuration
        // 
        this.grpPassRun = new Group(parent, SWT.NONE);
        this.grpPassRun.setText("Breakpoint in the passing run");
        this.grpPassRun.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        layout = new GridLayout(2, false);
        this.grpPassRun.setLayout(layout);

        this.createPassBPSection(this.grpPassRun);
    }

    private void createPassBPSection(Group parent) {
        // class name
        this.addLabel(parent, "Class Name:", 5);

        Composite compBPClassName = new Composite(parent, SWT.NONE);
        GridLayout gl = new GridLayout(2, false);
        gl.marginHeight = gl.marginWidth = 0;
        compBPClassName.setLayout(gl);
        compBPClassName.setFont(parent.getFont());
        compBPClassName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        this.txtPassBPClassName = new Text(compBPClassName, SWT.SINGLE
                | SWT.BORDER);
        this.txtPassBPClassName.setLayoutData(new GridData(
                GridData.FILL_HORIZONTAL));
        this.txtPassBPClassName.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                updateLaunchConfigurationDialog();
            }
        });

        this.butPassSearch = new Button(compBPClassName, SWT.PUSH);
        this.butPassSearch.setText("Search..."); //$NON-NLS-1$
        this.butPassSearch.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent evt) {
                String typeName = handleSearchButtonSelected();
                if (typeName != null) {
                    MatchpointTab.this.txtPassBPClassName.setText(typeName);
                }
            }
        });

        // line number
        this.addLabel(parent, "Line Number:", 5);

        this.txtPassBPLineNumber = new Text(parent, SWT.SINGLE | SWT.BORDER);
        this.txtPassBPLineNumber.setLayoutData(new GridData(
                GridData.FILL_HORIZONTAL));
        this.txtPassBPLineNumber.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent evt) {
                updateLaunchConfigurationDialog();
            }
        });

        // hit count
        this.addLabel(parent, "Hit Count:", 5);

        this.txtPassBPHitCount = new Text(parent, SWT.SINGLE | SWT.BORDER);
        this.txtPassBPHitCount.setLayoutData(new GridData(
                GridData.FILL_HORIZONTAL));
        this.txtPassBPHitCount.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent evt) {
                updateLaunchConfigurationDialog();
            }
        });
    }

    private void addLabel(Composite parent, String text, int horizontalIndent) {
        this.addLabel(parent, text, horizontalIndent, 1);
    }

    private void addLabel(Composite parent, String text, int horizontalIndent,
            int horizontalSpan) {
        Label label = new Label(parent, SWT.NONE);
        label.setText(text);
        GridData gd = new GridData(GridData.BEGINNING);
        gd.horizontalIndent = horizontalIndent;
        gd.horizontalSpan = horizontalSpan;
        label.setLayoutData(gd);
    }

    /**
     * Show a dialog that lists all types
     */
    String handleSearchButtonSelected() {
        String result = null;

        Shell parent = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
                .getShell();
        // begin fix https://bugs.eclipse.org/bugs/show_bug.cgi?id=66436
        OpenTypeSelectionDialog2 dialog;
        try {
            dialog = new OpenTypeSelectionDialog2(parent, false, PlatformUI
                    .getWorkbench().getProgressService(),
                     SearchEngine
                            .createWorkspaceScope(), IJavaSearchConstants.TYPE);
        } catch (OperationCanceledException e) {
            // action got canceled
            return null;
        }

        dialog.setTitle("Open TypeNode"); //$NON-NLS-1$
        dialog
                .setMessage("Select a type to open (? = any character, * = any string):"); //$NON-NLS-1$
        int returnCode = dialog.open();
        if (returnCode != IDialogConstants.OK_ID)
            return null;

        Object[] types = dialog.getResult();
        if (types != null && types.length > 0) {
            IType type = (IType) types[0];
            result = type.getFullyQualifiedName();
        }

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.debug.ui.AbstractLaunchConfigurationTab#updateLaunchConfigurationDialog()
     */
    protected void updateLaunchConfigurationDialog() {
        validatePage();
        super.updateLaunchConfigurationDialog();
    }

    /**
     * @see org.eclipse.debug.ui.ILaunchConfigurationTab#setDefaults(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
     */
    public void setDefaults(ILaunchConfigurationWorkingCopy configuration) {
        configuration.setAttribute(PASSING_BP_LINE_NUMBER, -1);
        configuration.setAttribute(PASSING_BP_HIT_COUNT, 1);

        configuration.setAttribute(FAILING_BP_LINE_NUMBER, -1);
        configuration.setAttribute(FAILING_BP_HIT_COUNT, 1);
    }

    /**
     * @see org.eclipse.debug.ui.ILaunchConfigurationTab#initializeFrom(org.eclipse.debug.core.ILaunchConfiguration)
     */
    public void initializeFrom(ILaunchConfiguration configuration) {

        int passingBPLine = -1;
        int failingBPLine = -1;
        int passingBPHitCount = 1;
        int failingBPHitCount = 1;

        //
        // passing breakpoint
        //
        try {
            this.txtPassBPClassName.setText(configuration.getAttribute(
                    PASSING_BP_CLASS_NAME, EMPTY_STRING));
        } catch (CoreException e) {
            // do nothing
        }

        String strResult = EMPTY_STRING;
        try {
            passingBPLine = configuration.getAttribute(PASSING_BP_LINE_NUMBER,
                    -1);
            if (passingBPLine >= 0) {
                strResult = String.valueOf(passingBPLine);

            }
        } catch (CoreException doNothing) {
            // do nothing
        }
        this.txtPassBPLineNumber.setText(strResult);

        strResult = EMPTY_STRING;
        try {
            passingBPHitCount = configuration.getAttribute(
                    PASSING_BP_HIT_COUNT, 1);
            if (passingBPHitCount >= 0) {
                strResult = String.valueOf(passingBPHitCount);

            }
        } catch (CoreException doNothing) {
            // do nothing
        }
        this.txtPassBPHitCount.setText(strResult);

        //
        // failing breakpoint
        //
        try {
            this.txtFailBPClassName.setText(configuration.getAttribute(
                    FAILING_BP_CLASS_NAME, EMPTY_STRING));
        } catch (CoreException e) {
            // do nothing
        }

        strResult = EMPTY_STRING;
        try {
            failingBPLine = configuration.getAttribute(FAILING_BP_LINE_NUMBER,
                    -1);
            if (failingBPLine >= 0) {
                strResult = String.valueOf(failingBPLine);
            }
        } catch (CoreException doNothing) {
            // do nothing
        }
        this.txtFailBPLineNumber.setText(strResult);

        strResult = EMPTY_STRING;
        try {
            failingBPHitCount = configuration.getAttribute(
                    FAILING_BP_HIT_COUNT, 1);
            if (failingBPHitCount >= 0) {
                strResult = String.valueOf(failingBPHitCount);
            }
        } catch (CoreException doNothing) {
            // do nothing
        }
        this.txtFailBPHitCount.setText(strResult);

        boolean enableMatchPointFinder = true;
        try {
            enableMatchPointFinder = configuration.getAttribute(ENABLE_MATCHPOINT_FINDER, true);
        } catch (CoreException e) {
            // do nothing
        }

        this.butEnableMatchPointFinder.setSelection(enableMatchPointFinder);
        this.setFormEnabled();
        this.validatePage();
    }

    public void activated(ILaunchConfigurationWorkingCopy workingCopy) {
        // do nothing
    }

    void setFormEnabled() {
        boolean enable = this.butEnableMatchPointFinder.getSelection();

        this.txtFailBPClassName.setEnabled(!enable);
        this.txtPassBPClassName.setEnabled(!enable);
        this.txtFailBPHitCount.setEnabled(!enable);
        this.txtPassBPHitCount.setEnabled(!enable);
        this.txtFailBPLineNumber.setEnabled(!enable);
        this.txtPassBPLineNumber.setEnabled(!enable);
        this.butFailSearch.setEnabled(!enable);
        this.butPassSearch.setEnabled(!enable);
    }

    /**
     * @see org.eclipse.debug.ui.ILaunchConfigurationTab#performApply(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
     */
    public void performApply(ILaunchConfigurationWorkingCopy configuration) {
        String passingBPClassName = this.txtPassBPClassName.getText().trim();
        String failingBPClassName = this.txtFailBPClassName.getText().trim();
        String passingBPLine = this.txtPassBPLineNumber.getText().trim();
        String failingBPLine = this.txtFailBPLineNumber.getText().trim();
        String passingBPHitCount = this.txtPassBPHitCount.getText().trim();
        String failingBPHitCount = this.txtFailBPHitCount.getText().trim();

        //
        // passing breakpoint
        //
        if (!passingBPClassName.equals(EMPTY_STRING)) {
            configuration.setAttribute(PASSING_BP_CLASS_NAME,
                    passingBPClassName);
        }

        if (!passingBPLine.equals(EMPTY_STRING)) {
            // try to parse to int
            try {
                configuration.setAttribute(PASSING_BP_LINE_NUMBER, new Integer(
                        passingBPLine).intValue());
            } catch (NumberFormatException doNothing) {
                // doNothing
            }
        }

        if (!passingBPHitCount.equals(EMPTY_STRING)) {
            // try to parse to int
            try {
                configuration.setAttribute(PASSING_BP_HIT_COUNT, new Integer(
                        passingBPHitCount).intValue());
            } catch (NumberFormatException doNothing) {
                // doNothing
            }
        }

        //
        // failing breakpoint
        //
        if (!failingBPClassName.equals(EMPTY_STRING)) {
            configuration.setAttribute(FAILING_BP_CLASS_NAME,
                    failingBPClassName);
        }

        if (!failingBPLine.equals(EMPTY_STRING)) {
            // try to parse to int
            try {
                configuration.setAttribute(FAILING_BP_LINE_NUMBER, new Integer(
                        failingBPLine).intValue());
            } catch (NumberFormatException doNothing) {
                // doNothing
            }
        }

        if (!failingBPHitCount.equals(EMPTY_STRING)) {
            // try to parse to int
            try {
                configuration.setAttribute(FAILING_BP_HIT_COUNT, new Integer(
                        failingBPHitCount).intValue());
            } catch (NumberFormatException doNothing) {
                // doNothing
            }
        }

        configuration.setAttribute(ENABLE_MATCHPOINT_FINDER,
                this.butEnableMatchPointFinder.getSelection());
    }

    /**
     * @see org.eclipse.debug.ui.ILaunchConfigurationTab#getName()
     */
    public String getName() {
        return "Matchpoint";
    }

    public IJavaProject getJavaProjectByName(String name) {
        IJavaProject result = null;

        if (!name.equals(EMPTY_STRING)) {
            IWorkspace workspace = ResourcesPlugin.getWorkspace();
            IWorkspaceRoot root = workspace.getRoot();
            IProject project = root.getProject(name);

            IJavaProject javaProject = JavaCore.create(project);
            if (javaProject != null && javaProject.exists()) {
                result = javaProject;
            }
        }

        return result;
    }

    public boolean isValid(ILaunchConfiguration launchConfig) {
        return this.getErrorMessage() == null;
    }

    void validatePage() {
        setErrorMessage(null);

        if (this.butEnableMatchPointFinder.getSelection())
            return;

        if (this.txtPassBPClassName.getText().trim().equals(EMPTY_STRING)) {
            setErrorMessage(ERR_MSG_PASS_BP_CLASS_NAME);
            return;
        }

        if (this.txtFailBPClassName.getText().trim().equals(EMPTY_STRING)) {
            setErrorMessage(ERR_MSG_FAIL_BP_CLASS_NAME);
            return;
        }

        String passBPLineNumber = this.txtPassBPLineNumber.getText().trim();
        if (passBPLineNumber.equals(EMPTY_STRING)
                || !isValidInteger(passBPLineNumber, new Integer(1))) {
            setErrorMessage(ERR_MSG_PASS_BP_LINE_NUMBER);
            return;
        }

        String failBPLineNumber = this.txtFailBPLineNumber.getText().trim();
        if (failBPLineNumber.equals(EMPTY_STRING)
                || !isValidInteger(failBPLineNumber, new Integer(1))) {
            setErrorMessage(ERR_MSG_FAIL_BP_LINE_NUMBER);
            return;
        }

        String passBPHitCount = this.txtPassBPHitCount.getText().trim();
        if (passBPHitCount.equals(EMPTY_STRING)
                || !isValidInteger(passBPHitCount, new Integer(1))) {
            setErrorMessage(ERR_MSG_PASS_BP_HIT_COUNT);
            return;
        }

        String failBPHitCount = this.txtFailBPHitCount.getText().trim();
        if (failBPHitCount.equals(EMPTY_STRING)
                || !isValidInteger(failBPHitCount, new Integer(1))) {
            setErrorMessage(ERR_MSG_FAIL_BP_HIT_COUNT);
            return;
        }
    }

    private boolean isValidInteger(String number, Integer minValue) {
        boolean result = true;

        try {
            int intNumber = Integer.parseInt(number);

            if (minValue != null) {
                result = intNumber >= minValue.intValue();
            }
        } catch (NumberFormatException e) {
            result = false;
        }

        return result;
    }

    public Image getImage() {
        return DDStateLaunchImages.getImage(DDStateLaunchImages.IMG_KEY_MATCHPOINT);
    }
}
