/*
 * $Id: DDRunResultPage.java,v 1.30 2006/01/29 13:25:56 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 19.04.2005 16:08:02 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.ui.views;

import java.util.Iterator;
import java.util.List;

import org.deltadebugging.ddstate.launcher.DDStateLaunchPlugin;
import org.deltadebugging.ddstate.launcher.client.IJUnitTestDescription;
import org.deltadebugging.ddstate.launcher.client.ITestDescription;
import org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult;
import org.deltadebugging.ddstate.launcher.ddrun.IDDTestResult;
import org.deltadebugging.ddstate.launcher.matcher.breakpoint.IBreakpointLocation;
import org.deltadebugging.ddstate.launcher.matcher.breakpoint.ILineBreakpointLocation;
import org.deltadebugging.ddstate.launcher.matcher.breakpoint.IMethodEntryBreakpointLocation;
import org.deltadebugging.ddstate.logger.DDLogger;
import org.deltadebugging.ddstate.memorygraph.IGraphEdge;
import org.deltadebugging.ddstate.memorygraph.IMethodIdentifier;
import org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta;
import org.deltadebugging.ddstate.memorygraph.delta.location.PathLocation;
import org.deltadebugging.ddstate.memorygraph.util.Path;
import org.deltadebugging.ddstate.ui.DDStateUIPlugin;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.Signature;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.events.HyperlinkAdapter;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.widgets.Hyperlink;
import org.eclipse.ui.part.IPageBookViewPage;
import org.eclipse.ui.part.IPageSite;

/**
 * COMMENTME Comment type DDRunResultPage
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class DDRunResultPage implements IPageBookViewPage {

	/** Personal logger for this class */
	final static DDLogger logger = DDStateUIPlugin
			.getLogger(DDRunResultPage.class);

	// page site
	private IPageSite fSite = null;

	private IDDRunResultView fView = null;

	private IDDRunResult fDDRunResult = null;

	// swt components
	Composite fMainControl = null;

	/**
	 * Constructs a new process page
	 */
	public DDRunResultPage(IDDRunResultView view, IDDRunResult ddRunResult) {
		this.fView = view;
		this.fDDRunResult = ddRunResult;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.IPageBookViewPage#getSite()
	 */
	public IPageSite getSite() {
		return this.fSite;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.IPageBookViewPage#init(org.eclipse.ui.part.IPageSite)
	 */
	public void init(IPageSite site) throws PartInitException {
		this.fSite = site;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.IPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		this.fMainControl = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		this.fMainControl.setLayout(layout);
		this.fMainControl.setLayoutData(new GridData(GridData.FILL_BOTH));
		this.fMainControl.setFont(parent.getFont());

		this.buildPage(this.fMainControl);
	}

	private void buildPage(Composite parent) {
		if (this.fDDRunResult.hasErrorStatus()) {
			this.createErrorEntries(this.fMainControl);
		}

		if (this.fDDRunResult.hasInfoStatus()) {
			this.createInfoEntries(this.fMainControl);
		}

		TabFolder tabFolder = new TabFolder(parent, SWT.TOP);
		tabFolder.setLayoutData(new GridData(GridData.FILL_BOTH));

		//
		// minimized deltas
		//
		this.createMinimizedDeltaTab(tabFolder);

		//
		// computed deltas
		//
		this.createComputedDeltaTab(tabFolder);

		//
		// delta history
		//
		this.createDeltaHistoryTab(tabFolder);

		//
		// information
		//
		this.createInfoTab(tabFolder);
	}

	private void createInfoTab(TabFolder tabFolder) {
		TabItem tabItLocation = new TabItem(tabFolder, SWT.NULL);
		tabItLocation.setText("&Info");
		tabItLocation
				.setToolTipText("Comparable locations of the two test runs.");

		Composite comp = new Composite(tabFolder, SWT.NULL);
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.grabExcessHorizontalSpace = true;
		comp.setLayoutData(gd);
		tabItLocation.setControl(comp);

		comp.setLayout(new GridLayout(3, false));

		ITestDescription failTestDesc = this.fDDRunResult.getDDRunDescription()
				.getFailTestDescription();
		ITestDescription passTestDesc = this.fDDRunResult.getDDRunDescription()
				.getPassTestDescription();

		IJUnitTestDescription failJUnitTestDesc = failTestDesc
				.getJUnitTestDescription();
		IJUnitTestDescription passJUnitTestDesc = passTestDesc
				.getJUnitTestDescription();

		String className = null;
		String methodName = null;
		String[] paramTypeSignature = new String[0];
		IJavaProject javaProject = null;

		//
		// title
		//
		this.createLabel(comp, "", GridData.BEGINNING, 1);
		this.createLabel(comp, "Passing Test", GridData.CENTER, 1);
		this.createLabel(comp, "Failing Test", GridData.CENTER, 1);

		// 
		// test method
		//
		this.createLabel(comp, "Test Method:", GridData.BEGINNING, 1);
		javaProject = passTestDesc.getJavaProject();
		className = passJUnitTestDesc.getTestCaseClassName();
		methodName = passJUnitTestDesc.getTestName();
		String labelText = FormTextUtil.cutPackageOfTypeName(className) + "."
				+ methodName + "()";
		this.createMethodLink(comp, labelText, javaProject, className,
				methodName, paramTypeSignature);

		javaProject = failTestDesc.getJavaProject();
		className = failJUnitTestDesc.getTestCaseClassName();
		methodName = failJUnitTestDesc.getTestName();
		labelText = FormTextUtil.cutPackageOfTypeName(className) + "."
				+ methodName + "()";
		this.createMethodLink(comp, labelText, javaProject, className,
				methodName, paramTypeSignature);

		//
		// matchpoint
		// 
		this.createLabel(comp, "Match Point:", GridData.BEGINNING, 1);
		this.createBreakpointInfo(comp, passTestDesc);
		this.createBreakpointInfo(comp, failTestDesc);
		
//		// empty label
//		this.createLabel(comp, "", GridData.FILL_HORIZONTAL, 3);
//
//		//
//		// Statistics
//		//
//		this.createLabel(comp, "Statistics", GridData.CENTER, 3);

	}

	private void createBreakpointInfo(Composite parent,
			ITestDescription testDesc) {
		IBreakpointLocation bpLocation = testDesc.getBreakpointLocation();

		if (bpLocation instanceof IMethodEntryBreakpointLocation) {
			IMethodEntryBreakpointLocation methodBPLocation = (IMethodEntryBreakpointLocation) bpLocation;
			IMethodIdentifier methodIdentifier = methodBPLocation
					.getMethodIdentifier();

			String strMethod = Signature.toString(methodIdentifier
					.getSignature(), methodIdentifier.getMethodName(), null,
					false, false);
			String strLocation = FormTextUtil.announceEnum(bpLocation
					.getHitCount())
					+ " call of method "
					+ strMethod
					+ " in "
					+ FormTextUtil.cutPackageOfTypeName(methodIdentifier
							.getTypeName());

			this
					.createMethodLink(parent, strLocation, testDesc
							.getJavaProject(), methodBPLocation.getTypeName(),
							methodIdentifier.getMethodName(), Signature
									.getParameterTypes(methodIdentifier
											.getSignature()));

		} else if (bpLocation instanceof ILineBreakpointLocation) {
			ILineBreakpointLocation lineBPLocation = (ILineBreakpointLocation) bpLocation;

			String strLocation = FormTextUtil.announceEnum(bpLocation
					.getHitCount())
					+ " call of line "
					+ lineBPLocation.getLineNumber()
					+ " in "
					+ FormTextUtil.cutPackageOfTypeName(bpLocation
							.getTypeName());

			this.createLineLink(parent, strLocation, testDesc.getJavaProject(),
					lineBPLocation.getTypeName(), lineBPLocation
							.getLineNumber());
		}

	}

	private Label createLabel(Composite parent, String text, int gdStyle,
			int horizontalSpan) {
		GridData gd = new GridData(gdStyle);
		gd.horizontalSpan = horizontalSpan;

		return this.createLabel(parent, text, gd);
	}

	private Label createLabel(Composite parent, String text, GridData gd) {
		Label result = new Label(parent, SWT.NULL);
		result.setText(text);
		result.setLayoutData(gd);

		return result;
	}

	private Hyperlink createMethodLink(Composite parent, String text,
			final IJavaProject javaProject, final String className,
			final String methodName, final String[] parameterTypeSignature) {
		Hyperlink result = null;

		Hyperlink link = this.createHyperlink(parent, text);
		link.addHyperlinkListener(new HyperlinkAdapter() {
			public void linkActivated(HyperlinkEvent event) {
				FormTextUtil.openMethod(javaProject, className, methodName,
						parameterTypeSignature);
			}
		});

		return result;
	}

	private Hyperlink createLineLink(Composite parent, String text,
			final IJavaProject javaProject, final String className,
			final int lineNumber) {
		Hyperlink result = null;

		Hyperlink link = this.createHyperlink(parent, text);
		link.addHyperlinkListener(new HyperlinkAdapter() {
			public void linkActivated(HyperlinkEvent event) {
				FormTextUtil.openLine(javaProject, className, lineNumber);
			}
		});

		return result;
	}

	private Hyperlink createHyperlink(Composite parent, String text) {
		Hyperlink result = new Hyperlink(parent, SWT.NULL);
		result.setText(text);
		result.setUnderlined(true);
		result
				.setForeground(parent.getDisplay().getSystemColor(
						SWT.COLOR_BLUE));
		result.setBackground(parent.getBackground());

		return result;
	}

	private void createDeltaHistoryTab(TabFolder tabFolder) {
		TabItem tabItHistory = new TabItem(tabFolder, SWT.NULL);
		tabItHistory.setText("Test &History");
		tabItHistory.setToolTipText("Test History of the delta debugging run");

		SashForm safoHistory = new SashForm(tabFolder, SWT.HORIZONTAL);
		safoHistory.setLayoutData(new GridData(GridData.FILL_BOTH));
		tabItHistory.setControl(safoHistory);

		Table tabTestHistory = this.addHistoryTable(safoHistory);
		DeltaHistoryTableViewer tabViewTestHistory = new DeltaHistoryTableViewer(
				tabTestHistory);
		tabViewTestHistory.setDDRunResult(this.fDDRunResult);

		Table tabTestHistoryDeltas = this.addDeltaTable(safoHistory);
		final GraphDeltaTableViewer tabViewTestHistoryDeltas = new GraphDeltaTableViewer(
				tabTestHistoryDeltas);

		safoHistory.setWeights(new int[] { 30, 70 });
		safoHistory.layout();

		tabTestHistory.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				IDDTestResult result = (IDDTestResult) e.item.getData();

				if (result != null) {
					tabViewTestHistoryDeltas.setDeltaSet(result
							.getTestedDeltas());
				}
			}
		});
	}

	private void createComputedDeltaTab(TabFolder tabFolder) {
		TabItem tabItCompDeltas = new TabItem(tabFolder, SWT.NULL);
		tabItCompDeltas.setText("&Computed Deltas");
		tabItCompDeltas
				.setToolTipText("Computed deltas of the delta debugging run");
		Table tabCompDeltas = this.addDeltaTable(tabFolder);
		tabItCompDeltas.setControl(tabCompDeltas);

		GraphDeltaTableViewer tabViewCompDeltas = new GraphDeltaTableViewer(
				tabCompDeltas);
		tabViewCompDeltas.setDeltaSet(this.fDDRunResult.getComputedDeltaSet());
	}

	private void createMinimizedDeltaTab(TabFolder tabFolder) {
		TabItem tabItMinDeltas = new TabItem(tabFolder, SWT.NULL);
		tabItMinDeltas.setText("&Minimized Deltas");
		tabItMinDeltas
				.setToolTipText("Minimized deltas of the delta debugging run");
		Table tabMinDeltas = this.addDeltaTable(tabFolder);
		tabItMinDeltas.setControl(tabMinDeltas);

		GraphDeltaTableViewer tabViewMinDeltas = new GraphDeltaTableViewer(
				tabMinDeltas);
		tabViewMinDeltas.setDeltaSet(this.fDDRunResult.getMinimizedDeltaSet());
	}

	private void createErrorEntries(Composite parent) {
		Label lblErrorStatus = new Label(parent, SWT.NULL);

		String errorText = "Errors: \n";
		for (Iterator itErrors = this.fDDRunResult.getErrorStatus().iterator(); itErrors
				.hasNext();) {
			IStatus error = (IStatus) itErrors.next();
			errorText += error.getMessage();

			if (error.getException() != null) {
				errorText += " (Exc.-Message = '"
						+ error.getException().getMessage() + "'";
			}

			errorText += "\n";
		}

		lblErrorStatus.setText(errorText);
	}

	private void createInfoEntries(Composite parent) {
		Label lblInfoStatus = new Label(parent, SWT.NULL);

		String infoText = "Infos: \n";
		for (Iterator itInfos = this.fDDRunResult.getInfoStatus().iterator(); itInfos
				.hasNext();) {
			IStatus info = (IStatus) itInfos.next();
			infoText += info.getMessage();

			if (info.getException() != null) {
				infoText += " (Exc.-Message = '"
						+ info.getException().getMessage() + "'";
			}

			infoText += "\n";
		}

		lblInfoStatus.setText(infoText);
	}

	private Table addHistoryTable(Composite parent) {
		Table result = new Table(parent, SWT.SINGLE | SWT.FULL_SELECTION
				| SWT.H_SCROLL);
		result.setLayoutData(new GridData(GridData.FILL_BOTH));

		TableColumn column = new TableColumn(result, SWT.RIGHT, 0);
		column.setText("Serial Number"); //$NON-NLS-1$
		column.setWidth(50);

		column = new TableColumn(result, SWT.LEFT, 1);
		column.setText("Description"); //$NON-NLS-1$
		column.setWidth(160);

		column = new TableColumn(result, SWT.LEFT, 2);
		column.setText(""); //$NON-NLS-1$
		column.setWidth(40);

		return result;
	}

	private Table addDeltaTable(final Composite parent) {
		final Table result = new Table(parent, SWT.MULTI | SWT.FULL_SELECTION
				| SWT.H_SCROLL);
		result.setLayoutData(new GridData(GridData.FILL_BOTH));
		result.setHeaderVisible(true);
		result.setLinesVisible(true);

		TableColumn column = new TableColumn(result, SWT.LEFT, 0);
		column.setText("Location"); //$NON-NLS-1$
		column.setWidth(400);

		column = new TableColumn(result, SWT.LEFT, 1);
		column.setText("Value of Pass State"); //$NON-NLS-1$
		column.setWidth(200);

		column = new TableColumn(result, SWT.LEFT, 2);
		column.setText("Value of Fail State"); //$NON-NLS-1$
		column.setWidth(200);

		result.addMouseListener(new MouseListener() {

			public void mouseDoubleClick(MouseEvent e) {
				columnSelected(e);
			}

			public void mouseDown(MouseEvent e) {
				// do nothing
			}

			public void mouseUp(MouseEvent e) {
				// do nothing
			}

		});

		// Implement a "fake" tooltip
		final Listener labelListener = new Listener() {
			public void handleEvent(Event event) {
				Label label = (Label) event.widget;
				Shell shell = label.getShell();
				switch (event.type) {
				case SWT.MouseDown:
					Event e = new Event();
					e.item = (TableItem) label.getData("_TABLEITEM");
					// Assuming table is single select, set the selection as if
					// the mouse down event went through to the table
					result.setSelection(new TableItem[] { (TableItem) e.item });
					result.notifyListeners(SWT.Selection, e);
				// fall through
				case SWT.MouseExit:
					shell.dispose();
					break;
				}
			}
		};

		// Disable native tooltip
		result.setToolTipText("");

		Listener tableListener = new Listener() {
			Shell tip = null;

			Label label = null;

			public void handleEvent(Event event) {
				switch (event.type) {
				case SWT.Dispose:
				case SWT.KeyDown:
				case SWT.MouseMove: {
					if (this.tip == null)
						break;
					this.tip.dispose();
					this.tip = null;
					this.label = null;
					break;
				}
				case SWT.MouseHover: {
					Display display = DDStateUIPlugin.getStandardDisplay();

					TableItem item = result
							.getItem(new Point(event.x, event.y));
					if (item != null) {
						if (this.tip != null && !this.tip.isDisposed())
							this.tip.dispose();
						this.tip = new Shell(parent.getShell(), SWT.ON_TOP
								| SWT.TOOL);
						this.tip.setLayout(new FillLayout());
						this.label = new Label(this.tip, SWT.NONE);
						this.label.setForeground(display
								.getSystemColor(SWT.COLOR_INFO_FOREGROUND));
						this.label.setBackground(display
								.getSystemColor(SWT.COLOR_INFO_BACKGROUND));
						this.label.setData("_TABLEITEM", item);
						this.label.setText(getPathString(item));
						this.label.addListener(SWT.MouseExit, labelListener);
						this.label.addListener(SWT.MouseDown, labelListener);
						Point size = this.tip.computeSize(SWT.DEFAULT,
								SWT.DEFAULT);
						Rectangle rect = item.getBounds(0);
						Point pt = result.toDisplay(rect.x, rect.y);
						this.tip
								.setBounds(pt.x + 30, pt.y + 40, size.x, size.y);
						this.tip.setVisible(true);
					}
				}
				}
			}
		};

		result.addListener(SWT.Dispose, tableListener);
		result.addListener(SWT.KeyDown, tableListener);
		result.addListener(SWT.MouseMove, tableListener);
		result.addListener(SWT.MouseHover, tableListener);

		return result;
	}

	void columnSelected(MouseEvent e) {
		Table tab = (Table) e.getSource();

		if (tab.getSelectionCount() == 1) {
			TableItem tabItem = tab.getSelection()[0];
			Object tabData = tabItem.getData();

			if (tabData instanceof IMemoryGraphDelta) {
				IMemoryGraphDelta delta = (IMemoryGraphDelta) tabData;

				if (delta.getLocation() instanceof PathLocation) {
					Path path = ((PathLocation) delta.getLocation()).getPath();
					this.openPathInMessageBox(path);
				}
			}
		}
	}

	final String getPathString(TableItem tabItem) {
		String result = "";

		if (tabItem != null) {
			Object tabData = tabItem.getData();

			if (tabData instanceof IMemoryGraphDelta) {
				IMemoryGraphDelta delta = (IMemoryGraphDelta) tabData;

				if (delta.getLocation() instanceof PathLocation) {
					Path path = ((PathLocation) delta.getLocation()).getPath();
					result = this.buildPathString(path);
				}
			}
		}

		return result;
	}

	/**
	 * Show the given status in an error dialog with the given title.
	 * 
	 * @param status
	 *            the status you want to show in the error dialog.
	 * @param title
	 *            the title of the error dialog
	 */
	private void openPathInMessageBox(Path path) {
		final String pathString = this.buildPathString(path);

		DDStateLaunchPlugin.getDisplay().asyncExec(new Runnable() {

			public void run() {
				MessageDialog.openInformation(DDStateLaunchPlugin
						.getWorkbenchWindow().getShell(), "Path to location",
						pathString);

			}
		});

	}

	private String buildPathString(Path path) {
		StringBuffer result = new StringBuffer();

		if (path != null) {
			List pathList = path.getAsList();

			result.append("Root");
			for (int i = 0; i < pathList.size(); i++) {
				IGraphEdge edge = path.get(i);
				result.append(" -> ");
				result.append(edge.getName());
			}
		}

		return result.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.IPage#dispose()
	 */
	public void dispose() {
		// do nothing
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.IPage#getControl()
	 */
	public Control getControl() {
		if (this.fMainControl != null) {
			return this.fMainControl;
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.IPage#setActionBars(org.eclipse.ui.IActionBars)
	 */
	public void setActionBars(IActionBars actionBars) {
		// do nothing
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.IPage#setFocus()
	 */
	public void setFocus() {
		// do nothing
	}

	/**
	 * Returns the view this page is contained in
	 * 
	 * @return the view this page is contained in
	 */
	protected IDDRunResultView getDDRunResultView() {
		return this.fView;
	}

	/**
	 * Returns the dd run result this page is displaying
	 * 
	 * @return the dd run result this page is displaying
	 */
	protected IDDRunResult getDDRunResult() {
		return this.fDDRunResult;
	}

}