/*
 * $Id: CTSResultView.java,v 1.17 2005/10/25 13:39:23 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Aug 17, 2005 1:16:41 PM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.ui.views.cts;

import org.deltadebugging.ddcore.DeltaSet;
import org.deltadebugging.ddstate.launcher.client.cts.CTSLauncher;
import org.deltadebugging.ddstate.launcher.client.cts.ICTSRunResult;
import org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult;
import org.deltadebugging.ddstate.launcher.matcher.breakpoint.IBreakpointLocation;
import org.deltadebugging.ddstate.launcher.matcher.breakpoint.ILineBreakpointLocation;
import org.deltadebugging.ddstate.launcher.matcher.breakpoint.IMethodEntryBreakpointLocation;
import org.deltadebugging.ddstate.memorygraph.IMethodIdentifier;
import org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta;
import org.deltadebugging.ddstate.ui.DDStateUIPlugin;
import org.deltadebugging.ddstate.ui.views.FormTextUtil;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.FormColors;
import org.eclipse.ui.forms.HyperlinkSettings;
import org.eclipse.ui.forms.widgets.FormText;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.eclipse.ui.part.ViewPart;

public class CTSResultView extends ViewPart {

    private FormToolkit fToolkit;

    private Font fBodyFont;

    private ScrolledForm fForm;

    private ICTSRunResult fCTSRunResult;

    private int fSerialNumber;

    private DeltaToRTextGenerator fRTextGenerator;

    private final static String COLOR_KEY_BLUE = DDStateUIPlugin.PLUGIN_ID
            + "BLUE";

    private final static String COLOR_KEY_ORANGE = DDStateUIPlugin.PLUGIN_ID
            + "ORANGE";

    private final static int EXC_NUMBER_SIZE = 30;

    private final static int DEFAULT_FONT_SIZE = 12;

    private final static int HEADLINE_FONT_SIZE = 18;

    private final static int CTS_FONT_SIZE = 8;

    /**
     * This is a callback that will allow us to create the viewer and initialize
     * it.
     */
    public void createPartControl(Composite parent) {
        this.fToolkit = new FormToolkit(parent.getDisplay());

        this.init();

        this.fForm = this.fToolkit.createScrolledForm(parent);
        this.setFontSizeForm();

        // this.fBodyFont = this.setFontSizeBody();

        this.fForm.setText("DDstate Debugging Status");

        this.fForm.setForeground(this.fToolkit.getColors().getColor(
                COLOR_KEY_BLUE));

        TableWrapLayout layout = new TableWrapLayout();

        this.fToolkit.getHyperlinkGroup().setHyperlinkUnderlineMode(
                HyperlinkSettings.UNDERLINE_HOVER);
        this.fToolkit.getHyperlinkGroup().setForeground(
                this.fToolkit.getColors().getColor(COLOR_KEY_ORANGE));
        this.fToolkit.getHyperlinkGroup().setActiveForeground(
                this.fToolkit.getColors().getColor(COLOR_KEY_ORANGE));

        this.fForm.getBody().setLayout(layout);
    }

    private void setFontSizeForm() {
        Font font = this.fForm.getFont();
        font = this.setFontData(font, this.fForm.getDisplay(),
                HEADLINE_FONT_SIZE, SWT.BOLD);
        this.fForm.setFont(font);
    }

    private Font setFontSizeBody() {
        Composite body = this.fForm.getBody();
        Font font = body.getFont();
        font = this.setFontData(font, body.getDisplay(), DEFAULT_FONT_SIZE,
                SWT.NORMAL);
        body.setFont(font);

        return font;
    }

    private Font setFontData(Font font, Device device, int size, int style) {
        FontData[] fontDatas = font.getFontData();
        for (int i = 0; i < fontDatas.length; i++) {
            fontDatas[i].setHeight(size);
            fontDatas[i].setStyle(style);
        }
        font = new Font(device, fontDatas);

        return font;
    }

    private Font setFontStyle(Font font, Device device, int style) {
        FontData[] fontDatas = font.getFontData();
        for (int i = 0; i < fontDatas.length; i++) {
            fontDatas[i].setStyle(style);
        }
        font = new Font(device, fontDatas);

        return font;
    }

    private Font setFontData(Font font, Device device, int size, int style,
            String fontName) {
        FontData[] fontDatas = font.getFontData();
        for (int i = 0; i < fontDatas.length; i++) {
            fontDatas[i].setHeight(size);
            fontDatas[i].setStyle(style);
            fontDatas[i].setName(fontName);
        }
        font = new Font(device, fontDatas);

        return font;
    }

    private void init() {
        FormColors formColors = this.fToolkit.getColors();
        formColors.createColor(COLOR_KEY_ORANGE, 247, 165, 33);
        // formColors.createColor(COLOR_KEY_ORANGE, 247, 0, 0);
        formColors.createColor(COLOR_KEY_BLUE, 85, 126, 153);
        // formColors.createColor(COLOR_KEY_BLUE, 0, 0, 153);

        this.fSerialNumber = 1;

        this.fRTextGenerator = new DeltaToRTextGenerator(this.fToolkit);
    }

    private void createLaunchInfo(Composite parent) {
        Composite comp = this.fToolkit.createComposite(parent, SWT.WRAP);
        comp.setFont(parent.getFont());
        TableWrapLayout layout = new TableWrapLayout();
        layout.leftMargin = 15;
        comp.setLayout(layout);

        FormText rtext = this.fToolkit.createFormText(comp, true);
        StringBuffer buf = new StringBuffer();
        buf.append(FormTextUtil.FORM_BEGIN);
        buf.append("This happens at your JUnit test if you invoke ");
        rtext.addHyperlinkListener(FormTextUtil.addMethodHyperLink(buf,
                getJavaProject(), getTestClassName(), getTestMethodName(),
                new String[0]));

        buf.append(" in ");
        rtext.addHyperlinkListener(FormTextUtil.addTypeHyperLink(buf,
                getJavaProject(), getTestClassName()));

        buf.append(FormTextUtil.FORM_END);

        rtext.setText(buf.toString(), true, false);
        rtext.setFont(comp.getFont());
    }

    private void addDDRunResult(Composite parent, IDDRunResult[] results) {
        Composite comp = this.fToolkit.createComposite(parent, SWT.WRAP);
        comp.setFont(parent.getFont());
        TableWrapLayout layout = new TableWrapLayout();
        layout.numColumns = 2;
        layout.leftMargin = 15;
        layout.horizontalSpacing = 20;
        comp.setLayout(layout);

        this.fRTextGenerator.setJavaProject(this.getJavaProject());

        Label lblInfo = this.fToolkit.createLabel(comp,
                "DDstate has finished debugging your program.", SWT.WRAP);
        lblInfo.setFont(comp.getFont());
        TableWrapData td = new TableWrapData();
        td.colspan = 2;
        lblInfo.setLayoutData(td);

        for (int i = 0; i < results.length; i++) {
            IDDRunResult ddRunResult = results[i];

            this.createNumber(comp);

            Composite deltaComp = this.fToolkit.createComposite(comp, SWT.WRAP);
            deltaComp.setFont(comp.getFont());
            layout = new TableWrapLayout();
            deltaComp.setLayout(layout);

            this.createBreakpointLocation(ddRunResult, deltaComp);

            if (i == 0) {
                Label lblSince = this.fToolkit.createLabel(deltaComp,
                        "Since the JUnit testcase "
                                + this.getTestMethodName()
                                + " in "
                                + FormTextUtil.cutPackageOfTypeName(this
                                        .getTestClassName()) + " was invoked,");
                lblSince.setFont(comp.getFont());
            }

            this.createDeltaSection(deltaComp, ddRunResult);

            if ((i + 1) < results.length) {
                if (CTSLauncher
                        .isCauseTransition(ddRunResult, results[(i + 1)])) {
                    this.createCTSSign(comp);
                }
            }
        }
    }

    private void createDeltaSection(Composite parent, IDDRunResult ddRunResult) {
        Composite comp = this.fToolkit.createComposite(parent, SWT.WRAP);
        comp.setFont(parent.getFont());
        TableWrapLayout layout = new TableWrapLayout();
        layout.leftMargin = 0;
        layout.topMargin = 0;
        layout.bottomMargin = 0;
        layout.rightMargin = 0;
        layout.verticalSpacing = 0;
        layout.horizontalSpacing = 0;
        comp.setLayout(layout);

        DeltaSet minDeltas = ddRunResult.getMinimizedDeltaSet();

        if (minDeltas != null) {
            IMemoryGraphDelta[] arrMinDeltas = (IMemoryGraphDelta[]) minDeltas
                    .toArray(new IMemoryGraphDelta[minDeltas.size()]);

            String suffix = "";
            String prefix = "";
            int length = arrMinDeltas.length;

            for (int i = 0; i < length; i++) {

                IMemoryGraphDelta delta = arrMinDeltas[i];

                if (length > 1 && i == length - 2) {
                    suffix = " and";
                } else if (length > 2 && i < length - 1) {
                    suffix = ",";
                } else {
                    suffix = ".";
                }

                FormText deltaText = this.fRTextGenerator.createFormText(comp,
                        delta, prefix, suffix);
                deltaText.setFont(comp.getFont());
            }

            if (length == 0) {
                Label lblNoDelta = this.fToolkit.createLabel(comp,
                        "no relevant deltas were found.");
                lblNoDelta.setFont(comp.getFont());
            }
        }
    }

    private void createBreakpointLocation(IDDRunResult ddRunResult,
            Composite comp) {
        final IBreakpointLocation bpLocation = ddRunResult
                .getDDRunDescription().getFailTestDescription()
                .getBreakpointLocation();

        FormText rtext = this.fToolkit.createFormText(comp, true);

        StringBuffer buf = new StringBuffer();
        buf.append(FormTextUtil.FORM_BEGIN);

        buf.append("Execution reaches ");

        if (bpLocation instanceof ILineBreakpointLocation) {
            rtext.addHyperlinkListener(FormTextUtil.addLineHyperLink(buf,
                    getJavaProject(), bpLocation.getTypeName(),
                    ((ILineBreakpointLocation) bpLocation).getLineNumber()));

        } else if (bpLocation instanceof IMethodEntryBreakpointLocation) {
            IMethodEntryBreakpointLocation methodBPLocation = (IMethodEntryBreakpointLocation) bpLocation;
            IMethodIdentifier methodIdentifier = methodBPLocation
                    .getMethodIdentifier();

            rtext.addHyperlinkListener(FormTextUtil.addMethodHyperLink(buf,
                    getJavaProject(), bpLocation.getTypeName(),
                    methodIdentifier.getMethodName(), methodIdentifier
                            .getSignature()));
        }

        buf.append(" in ");
        rtext.addHyperlinkListener(FormTextUtil.addTypeHyperLink(buf,
                getJavaProject(), bpLocation.getTypeName()));

        buf
                .append(" for the "
                        + FormTextUtil.announceEnum(bpLocation.getHitCount())
                        + " time");

        buf.append(FormTextUtil.FORM_END);

        rtext.setText(buf.toString(), true, false);
        rtext.setForeground(this.fToolkit.getColors().getColor(COLOR_KEY_BLUE));

        Font font = this.setFontStyle(comp.getFont(), comp.getDisplay(),
                SWT.BOLD);
        rtext.setFont(font);
    }

    private void createNumber(Composite parent) {
        Label lblNumber = this.fToolkit.createLabel(parent, ""
                + this.fSerialNumber++);

        Font font = this.setFontData(parent.getFont(), parent.getDisplay(),
                EXC_NUMBER_SIZE, SWT.BOLD);

        lblNumber.setFont(font);
        lblNumber.setForeground(this.fToolkit.getColors().getColor(
                COLOR_KEY_ORANGE));
        TableWrapData td = new TableWrapData(TableWrapData.RIGHT);
        lblNumber.setLayoutData(td);
    }

    private void createCTSSign(Composite parent) {
        Label lblCTS = this.fToolkit.createLabel(parent, "> CTS <");

        Font font = this.setFontData(parent.getFont(), parent.getDisplay(),
                CTS_FONT_SIZE, SWT.BOLD);

        lblCTS.setFont(font);
        lblCTS
                .setForeground(this.fToolkit.getColors().getColor(
                        COLOR_KEY_BLUE));
        TableWrapData td = new TableWrapData(TableWrapData.CENTER);
        lblCTS.setLayoutData(td);

        Label lblEmpty = this.fToolkit.createLabel(parent, "");

        lblEmpty.setFont(font);

    }

    IJavaProject getJavaProject() {
        IJavaProject result = null;

        if (this.fCTSRunResult != null) {
            result = this.fCTSRunResult.getLaunchDescription().getJavaProject();
        }

        return result;
    }

    String getTestMethodName() {
        String result = null;

        if (this.fCTSRunResult != null) {
            result = this.fCTSRunResult.getLaunchDescription()
                    .getFailJUnitTestDescription().getTestName();
        }

        return result;
    }

    String getTestClassName() {
        String result = null;

        if (this.fCTSRunResult != null) {
            result = this.fCTSRunResult.getLaunchDescription()
                    .getFailJUnitTestDescription().getTestCaseClassName();
        }

        return result;
    }

    /**
     * Passing the focus request to the form.
     */
    public void setFocus() {
        this.fForm.setFocus();
    }

    /**
     * Disposes the toolkit.
     */
    public void dispose() {
        this.fToolkit.dispose();
        super.dispose();
    }

    public void display(ICTSRunResult result) {
        this.fCTSRunResult = result;
        this.setPartName("DDState Status - "
                + result.getLaunchDescription().getDDLaunchID());
        this.createLaunchInfo(this.fForm.getBody());

        this.addDDRunResult(this.fForm.getBody(), this.fCTSRunResult
                .getAllCauseTransitions());
        this.fForm.redraw();
    }

}
