/*
 * $Id: GraphVisualizer.java,v 1.12 2005/09/23 08:57:29 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 26.08.2004 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.visualizer;

import java.util.Iterator;
import java.util.Vector;

import org.deltadebugging.ddstate.memorygraph.filter.nodefilter.GraphNodeTypeFilter;
import org.deltadebugging.ddstate.visualizer.attributes.GraphVizAttribute;
import org.deltadebugging.ddstate.visualizer.attributes.GraphVizAttributeList;
import org.deltadebugging.ddstate.visualizer.vizgraph.VizGraphEdge;
import org.deltadebugging.ddstate.visualizer.vizgraph.VizGraphNode;
import org.deltadebugging.ddstate.visualizer.vizgraph.exceptions.GraphVisualizerException;

/**
 * This class can be use to visualize a memory graph. This class use the java
 * library to GraphViz tool to write a file, which can be visualized with dotty.
 * Dotty is an element of GraphViz tools.
 * 
 * @see <a href="http://www.graphviz.org/">GraphViz - Graph Drawing Tools </a>
 * @see <a href="http://www.research.att.com/~john/Grappa/"> Grappa - A Java
 *      Graph Package for GraphViz </a>
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class GraphVisualizer extends AbstractGraphVisualizer {

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer#getOutputDirectory()
     */
    public String getOutputDirectory() {
        return getPluginPreferences().getString(
                IVisualizerPreferenceConstants.GRAPHVIZ_MEMGRAPH_PATH);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer#showFullQualifiedNames()
     */
    public boolean showFullQualifiedNames() {
        return getPluginPreferences()
                .getBoolean(
                        IVisualizerPreferenceConstants.GRAPHVIZ_SHOW_FULL_QUALIFIED_NAMES);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer#showCountForHiddenNodes()
     */
    public boolean showCountForHiddenNodes() {
        return true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer#getGraphFilters()
     */
    public Vector getGraphFilters() {
        Vector filters = new Vector();

        // filter types
        boolean filterTypes = false;

        filterTypes = getPluginPreferences().getBoolean(
                IVisualizerPreferenceConstants.GRAPHVIZ_FILTER_TYPES);

        if (filterTypes)
            filters.add(new GraphNodeTypeFilter());

        return filters;
    }

    /*
     * @see AbstractGraphVisualizer#getNodeContent(VizGraphNode, boolean)
     */
    public String getNodeContent(VizGraphNode graphNode,
            boolean showFullQualifiedNames) throws GraphVisualizerException {
        return graphNode.getGraphVizName(showFullQualifiedNames);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer#getNodeAttributes(org.deltadebugging.ddstate.memorygraph.IGraphNode)
     */
    public Iterator getNodeAttributes(VizGraphNode graphNode) {
        GraphVizAttributeList result = graphNode.getGraphVizAttributes();
        
        if (PRESENTATION) {
            result.addAttribute(GraphVizAttribute.FONTCOLOR_WHITE);
        }
        
        return result.iterator();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer#getEdgeContent(org.deltadebugging.ddstate.memorygraph.IGraphEdge,
     *      boolean)
     */
    public String getEdgeContent(VizGraphEdge graphEdge,
            boolean showFullQualifiedNames) {
        return graphEdge.getGraphVizName(showFullQualifiedNames);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer#getEdgeAttributes(org.deltadebugging.ddstate.memorygraph.IGraphEdge)
     */
    public Iterator getEdgeAttributes(VizGraphEdge graphEdge) {
        GraphVizAttributeList result = graphEdge.getGraphVizAttributes();
        
        if (PRESENTATION) {
            result.addAttribute(GraphVizAttribute.COLOR_WHITE);
            result.addAttribute(GraphVizAttribute.FONTCOLOR_WHITE);
        }
        
        return result.iterator();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer#getAllSuccessors(org.deltadebugging.ddstate.memorygraph.IGraphNode)
     */
    public Iterator getAllSuccessors(VizGraphNode graphNode) {
        return graphNode.getSuccessors().iterator();
    }

}