/*
 * $Id: IVizGraphElement.java,v 1.1 2004/12/17 13:10:31 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 16.12.2004 09:41:04 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.visualizer.vizgraph;

import org.deltadebugging.ddstate.memorygraph.IGraphElement;
import org.deltadebugging.ddstate.visualizer.attributes.GraphVizAttributeList;
import org.deltadebugging.ddstate.visualizer.vizgraph.exceptions.GraphVisualizerException;

/**
 * The IVizGraphElement is a decorator for the graph elements.
 * 
 * @see org.deltadebugging.ddstate.memorygraph.IGraphElement
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface IVizGraphElement {

    /**
     * Return the graph element of this decorator.
     * 
     * @return the graph element of this decorator.
     */
    public IGraphElement getGraphElement();

    /**
     * Return the name of the element to visualize.
     * 
     * @param fullQualified
     *            if this is true the function return the full qualified name of
     *            the graph element (z.B. java.lang.String) else the
     *            abbreviation (z.B. String).
     * 
     * @return the name of the element.
     */
    public String getGraphVizName(boolean fullQualified) throws GraphVisualizerException;

    /**
     * Return a GraphVizAttributeList containing all attributes which would use
     * to visualize the specified element.
     * 
     * @return an GraphVizAttributeList.
     */
    public GraphVizAttributeList getGraphVizAttributes() throws GraphVisualizerException;
}

