/*
 * $Id: VizGraphElementFactory.java,v 1.7 2005/11/21 19:09:38 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 15.03.2005 11:05:14 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.visualizer.vizgraph;

import org.deltadebugging.ddstate.memorygraph.IGraphEdge;
import org.deltadebugging.ddstate.memorygraph.IGraphNode;
import org.deltadebugging.ddstate.memorygraph.IRootNode;
import org.deltadebugging.ddstate.memorygraph.type.IPrimitiveTypeNode;
import org.deltadebugging.ddstate.memorygraph.type.IReferenceTypeNode;
import org.deltadebugging.ddstate.memorygraph.value.IArrayNode;
import org.deltadebugging.ddstate.memorygraph.value.IClassObjectNode;
import org.deltadebugging.ddstate.memorygraph.value.INullValueNode;
import org.deltadebugging.ddstate.memorygraph.value.IObjectNode;
import org.deltadebugging.ddstate.memorygraph.value.IPrimitiveValueNode;
import org.deltadebugging.ddstate.memorygraph.value.IStringObjectNode;
import org.deltadebugging.ddstate.visualizer.vizgraph.type.VizPrimitiveTypeNode;
import org.deltadebugging.ddstate.visualizer.vizgraph.type.VizReferenceTypeNode;
import org.deltadebugging.ddstate.visualizer.vizgraph.value.VizArrayNode;
import org.deltadebugging.ddstate.visualizer.vizgraph.value.VizClassObjectNode;
import org.deltadebugging.ddstate.visualizer.vizgraph.value.VizNullValueNode;
import org.deltadebugging.ddstate.visualizer.vizgraph.value.VizObjectNode;
import org.deltadebugging.ddstate.visualizer.vizgraph.value.VizPrimitiveValueNode;
import org.deltadebugging.ddstate.visualizer.vizgraph.value.VizStringObjectNode;

/**
 * COMMENTME Comment type VizGraphElementFactory
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class VizGraphElementFactory {

    private VizGraphElementFactory() {
        // avoid constructing a new class with the default constructor
    }

    public final static VizGraphNode createGraphNode(IGraphNode graphNode,
            VizRootNode rootNode) {
        VizGraphNode result = null;

        result = (VizGraphNode) rootNode.alreadyRegister(graphNode);

        if (result == null) {
        	if (graphNode instanceof IClassObjectNode) {
                result = new VizClassObjectNode(
                        (IClassObjectNode) graphNode, rootNode);        		
        	} else if (graphNode instanceof IStringObjectNode) {
                result = new VizStringObjectNode(
                        (IStringObjectNode) graphNode, rootNode);
            } else if (graphNode instanceof INullValueNode) {
                result = new VizNullValueNode(
                        (INullValueNode) graphNode, rootNode);
            } else if (graphNode instanceof IPrimitiveValueNode) {
                result = new VizPrimitiveValueNode(
                        (IPrimitiveValueNode) graphNode, rootNode);
            } else if (graphNode instanceof IArrayNode) {
                result = new VizArrayNode(
                        (IArrayNode) graphNode, rootNode);
            } else if (graphNode instanceof IObjectNode) {
                result = new VizObjectNode(
                        (IObjectNode) graphNode, rootNode);
            } else if (graphNode instanceof IReferenceTypeNode) {
                result = new VizReferenceTypeNode(
                        (IReferenceTypeNode) graphNode, rootNode);
            } else if (graphNode instanceof IPrimitiveTypeNode) {
                result = new VizPrimitiveTypeNode(
                        (IPrimitiveTypeNode) graphNode, rootNode);
            } else if (graphNode instanceof IRootNode) {
                result = new VizRootNode((IRootNode) graphNode, rootNode);
            } else {
                throw new IllegalArgumentException(
                        "Method do not support the given graph node type - "
                                + graphNode.getClass().getName());
            }

            // we have to register the graph nodes
            rootNode.registerCreatedVizGraphElements(result);
        }

        return result;
    }

    // COMMENTME valueOf
    public final static VizGraphEdge createGraphEdge(IGraphEdge graphEdge,
            VizRootNode rootNode) {
        VizGraphEdge result = null;

        result = (VizGraphEdge) rootNode.alreadyRegister(graphEdge);

        if (result == null) {
            result = new VizGraphEdge(graphEdge, rootNode);

            // we have to register the graph edges
            rootNode.registerCreatedVizGraphElements(result);
        }

        return result;
    }

}