/*
 * $Id: IGraphComparator.java,v 1.2 2005/07/25 07:50:28 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 14.12.2004 11:33:58 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.comparator;

import org.deltadebugging.ddcore.DeltaSet;
import org.deltadebugging.ddstate.comparator.exceptions.GraphComparatorException;
import org.deltadebugging.ddstate.memorygraph.IRootNode;
import org.eclipse.core.runtime.IProgressMonitor;

/**
 * The graph comparator is a tool to compute the deltas of two memory graphs.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface IGraphComparator {

    /**
     * Compare the given two memory graphs and return the delta set as a set of
     * <code>IGraphElementDelta</code>. If we apply all computed deltas the
     * first graph must transfer to the second graph.
     * 
     * @param ddRunID
     *            the id of the current delta debugging run
     * @param firstGraph
     *            root node of the first graph to compare
     * @param secondGraph
     *            root node of the second graph to compare
     * @param monitor
     * @return a delta set of <code>IGraphElementDelta</code>
     * @throws GraphComparatorException
     *             if something failed on graph comparation
     * 
     * @see org.deltadebugging.ddstate.memorygraph.delta.IGraphElementDelta
     */
    public DeltaSet compare(int ddRunID, IRootNode firstGraph,
            IRootNode secondGraph, IProgressMonitor monitor)
            throws GraphComparatorException;

}
