/*
 * $Id: BaseGraphExtractor.java,v 1.1 2005/07/28 12:43:04 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 28.07.2005 14:35:01 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.extractor;

import java.util.Iterator;

import org.deltadebugging.ddstate.DDStatePlugin;
import org.deltadebugging.ddstate.IGraphExtractorListener;
import org.deltadebugging.ddstate.logger.DDLogger;
import org.deltadebugging.ddstate.memorygraph.IRootNode;
import org.eclipse.core.runtime.ISafeRunnable;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;

/**
 * The BaseGraphExtractor provide the functionality for notify all registered
 * listener that the graph is extracted.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public abstract class BaseGraphExtractor implements IGraphExtractor {

    /** Personal logger for this class */
    final static DDLogger logger = DDStatePlugin
            .getLogger(GraphExtractor.class);

    public abstract class ListenerSafeRunnable implements ISafeRunnable {
        public void handleException(Throwable t) {
            logger.log(IStatus.ERROR, "Exception on notify listener", t);
        }
    }

    /**
     * Notify all graph extraction listeners that the graph is extracted.
     * 
     * @param rootNode
     *            the root node of the extracted memory graph.
     */
    public void notifyGraphExtracted(final IRootNode rootNode) {
        logger.trace(DDStatePlugin.DEBUG_GRAPH_EXTRACTOR,
                "notify Listeners - graph extraced");

        for (Iterator itListener = DDStatePlugin.getDefault()
                .getGraphExtractorListeners().iterator(); itListener.hasNext();) {
            final IGraphExtractorListener listener = (IGraphExtractorListener) itListener
                    .next();

            Platform.run(new ListenerSafeRunnable() {
                public void run() {
                    listener.graphExtracted(rootNode);
                }
            });
        }
    }

}
