/*
 * $Id: IGraphExtractor.java,v 1.2 2005/08/22 07:25:00 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 28.07.2005 14:30:29 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.extractor;

import org.deltadebugging.ddstate.extractor.exceptions.GraphExtractorException;
import org.deltadebugging.ddstate.launcher.client.IJUnitTestDescription;
import org.deltadebugging.ddstate.memorygraph.IRootNode;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.debug.core.IJavaThread;

/**
 * The graph extractor create a memory graph at a given position on the programm
 * execution. This means you run your program until your preferred position is
 * reached and suspend the thread. After that you can call the graph extractor.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface IGraphExtractor {

	/**
	 * Create a new memory graph of the given thread. It is recommanded that the
	 * thread is suspended at the position where the memory graph should be
	 * extracted.
	 * 
	 * @param thread
	 * @param passingRun
	 *            set to true if you want to extract the graph of the passing
	 *            run, else false.
	 * @return a new memory graph.
	 * @throws GraphExtractorException
	 *             if the given thread is not suspended or an exception occured
	 *             on reading out the inforamtion of the memory.
	 * @throws IllegalArgumentException
	 *             if the given thread is null.
	 */
	public abstract IRootNode createMemoryGraph(int ddRunID,
			final IJavaThread thread, boolean passingRun,
			IJUnitTestDescription testDesc, IProgressMonitor monitor)
			throws GraphExtractorException;
}