/*
 * $Id: ILocalVariableEdge.java,v 1.1 2005/08/22 11:01:08 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 03.08.2005 14:17:45 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph;

/**
 * This edge joining the root node with a variable directly accessible from the
 * stack frames. This variables are the so called local variables of the
 * frame. In addition to ordinary graph edges this edge contains the stackframe,
 * which the local variable belongs to and the variable name.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface ILocalVariableEdge extends IGraphEdge {
    /**
     * Use this prefix to generate the edge name. The edge is build as follows:
     * <code>FRAME_PREFIX[$STACKFRAME_NUMBER].$VARIABLE_NAME</code>.
     */
    public static final String FRAME_PREFIX = "frame";

    /**
     * Returns the stack frame, which the local variable (target node) of this
     * edge belongs to.
     * 
     * @return the stack frame of the local variable.
     */
    public abstract IStackFrame getStackFrame();

    /**
     * Returns the name of the local variable (target node).
     * 
     * @return the name of the local variable.
     */
    public abstract String getVariableName();
}