/*
 * $Id: IRootNode.java,v 1.9 2005/11/16 15:23:46 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 15.12.2004 20:00:49 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph;

import org.deltadebugging.ddstate.memorygraph.exceptions.MemoryGraphExtractionException;
import org.deltadebugging.ddstate.memorygraph.value.IObjectNode;
import org.eclipse.jdt.debug.core.IJavaObject;
import org.eclipse.jdt.debug.core.IJavaType;
import org.eclipse.jdt.debug.core.IJavaVariable;

/**
 * The root node interface provides the additional functionality for the root
 * nodes. The root node has a special role in in graph. He act as anchor and
 * source node without predecessors. If you want to compare two memory graphs
 * you can start with the root node, because this are equal per definition.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface IRootNode extends IGraphNode {

    /**
     * Returns the container with some statistics values about this memory
     * graph.
     * 
     * @return the container with some statistics values about this memory
     *         graph.
     */
    public Statistics getStatistics();

    /**
     * Return true if this is the root node of the passing graph, else false.
     * 
     * @return true if this is the root node of the passing graph, else false.
     */
    public boolean isPassingGraph();

    /**
     * Return the unique delta debugging run id, which this root node belongs
     * to.
     * 
     * @return the unique delta debugging run id.
     */
    public int getDDRunID();

    /**
     * Look into the map for the object references if a ObjectNode exists for
     * the given objectReference.
     * 
     * @param objectReference
     *            the object reference specify the lookup key.
     * @return true if a GraphNode exists for the given objectReference else
     *         false.
     */
    public boolean existObjectNode(IJavaObject objectReference);

    /**
     * Return the ObjectNode for the given objectReference if one exists - else
     * return null.
     * 
     * @param objectReference
     *            the object reference specify the lookup key.
     * @return The Graphnode of the given key or null if no entry in the object
     *         reference map exists.
     * @throws IllegalArgumentException
     *             if the given objectReference is null.
     */
    public IObjectNode getObjectNode(IJavaObject objectReference);

    /**
     * Register a ObjectNode in the object reference map with the value of the
     * Graphnode as key.
     * 
     * @param graphNode
     *            the Graphnode to register.
     * @throws IllegalArgumentException
     *             if the Graphnode is null.
     */
    public void registerObjectNode(IObjectNode graphNode);

    /**
     * Creates a new successor for the given local variable and add this to this
     * root node.
     * 
     * @param var
     *            the java variable of the new successor
     * @param stackFrame
     *            the stackframe where the value belongs to
     * @throws MemoryGraphExtractionException
     *             if something went wrong on creating the graph node for the
     *             given java variable.
     */
    public void addLocalVariable(IJavaVariable var, IStackFrame stackFrame)
            throws MemoryGraphExtractionException;

    /**
     * Creates a new successor for the given type and add this to this root
     * node.
     * 
     * @param type
     *            the type of the new successor
     * @throws MemoryGraphExtractionException
     *             if something went wrong on creating the graph node for the
     *             given type.
     */
    public void addType(IJavaType type) throws MemoryGraphExtractionException;

}