/*
 * $Id: IStackFrame.java,v 1.3 2005/08/21 13:14:11 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Aug 2, 2005 3:25:20 PM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph;

import org.eclipse.jdt.debug.core.IJavaStackFrame;

/**
 * A stack frame in a thread on a Java virtual machine.
 * 
 * @see org.eclipse.jdt.debug.core.IJavaStackFrame
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface IStackFrame {
	/**
	 * Returns the number of this stack frame in the stack trace. The frame on
	 * the top of the stack frame and so the latest called method get the number
	 * 0.
	 * 
	 * @return number of this stack frame.
	 */
	public abstract int getFrameNumber();

	/**
	 * Returns the total number of all stack frames in the stack trace.
	 * 
	 * @return total number of all stack frames.
	 */
	public abstract int getFramesCount();

	/**
	 * Returns the method identifier that characterise the method associated
	 * with the stack frame.
	 * 
	 * @return the method identifier that characterise the method associated
	 *         with the stack frame.
	 */
	public abstract IMethodIdentifier getMethodIdentifier();

	/**
	 * Returns the line number of the instruction pointer in this stack frame
	 * that corresponds to a line in an associated source element, or
	 * <code>-1</code> if line number information is unavailable.
	 * 
	 * @return line number of instruction pointer in this stack frame, or
	 *         <code>-1</code> if line number information is unavailable
	 */
	public abstract int getLineNumber();

	/**
	 * Returns the underlying java stack frame of this stack frame wrapper
	 * class.
	 * 
	 * @return the underlying java stack frame.
	 */
	public abstract IJavaStackFrame getUnderlyingStackFrame();
}