/*
 * $Id: LocalVariableEdge.java,v 1.6 2005/11/28 15:28:44 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Aug 2, 2005 1:30:50 PM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph;

import org.deltadebugging.ddstate.memorygraph.value.IValueNode;

/**
 * This edge joining the root node with a variable directly accessible from the
 * stack frames. This variables are the so called local variables of the frame.
 * In addition to ordinary graph edges this edge contains the stackframe, which
 * the local variable belongs to and the variable name.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class LocalVariableEdge extends GraphEdge implements ILocalVariableEdge {

    private final IStackFrame fStackFrame;
    private final String fVariableName;

    /**
     * Create a new directed graph edge between the root node and a base
     * variable.
     * 
     * @param rootNode
     *            the root node of this graph element.
     * @param targetNode
     *            the target node where the edge points to.
     * @param fName
     *            the name of the edge. The name is the name of the operation by
     *            which you can obtain the target node from the source node.
     */
    public LocalVariableEdge(IRootNode rootNode, IValueNode targetNode,
            IStackFrame stackFrame, String variableName) {
        super(rootNode, targetNode, null);

        this.fStackFrame = stackFrame;
        this.fVariableName = variableName;

        this.fName = LocalVariableEdge.generatedEdgeName(this.fStackFrame
                .getFrameNumber(), this.fVariableName);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.memorygraph.ILocalVariableEdge#getStackFrame()
     */
    public IStackFrame getStackFrame() {
        return this.fStackFrame;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.memorygraph.ILocalVariableEdge#getVariableName()
     */
    public String getVariableName() {
        return this.fVariableName;
    }

    /**
     * Returns the edge name which is generated after the description in the
     * <code>IBaseVariableEdge</code> interface.
     * 
     * @param frameNumber
     *            the frame number of the underlying stackframe.
     * @param variableName
     *            the name of the base variable.
     * @return generated edge name.
     */
    public static String generatedEdgeName(int frameNumber, String variableName) {
        return FRAME_PREFIX + "[" + frameNumber + "]." + variableName;
    }
}
