/*
 * $Id: IMemoryGraphDelta.java,v 1.6 2005/11/21 10:42:04 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 29.07.2005 17:49:34 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.delta;

import java.util.Collection;

import org.deltadebugging.ddcore.DeltaSet;
import org.deltadebugging.ddstate.launcher.client.ITestRunner;
import org.deltadebugging.ddstate.memorygraph.delta.location.IDeltaLocation;
import org.deltadebugging.ddstate.memorygraph.exceptions.DeltaExecutionException;
import org.eclipse.jdt.debug.core.IJavaThread;

/**
 * The memory graph delta describes differences between two memory graphs. A
 * delta also supports methods to transfer one memory graph into another.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface IMemoryGraphDelta {

	public boolean ctsEqual(IMemoryGraphDelta delta);
	
    /**
     * Apply this delta. <br>
     * <b>Note: </b> You should have called prepare for this delta before you
     * call apply.
     * 
     * @param thread
     *            the thread of the program execution.
     * @param testRunner
     *            the test runner, which provides usefull support to create new
     *            objects.
     * @throws DeltaExecutionException
     *             if something wrent wrong on apply.
     */
    public void apply(IJavaThread thread, ITestRunner testRunner)
            throws DeltaExecutionException;


    /**
     * Add the given delta to the list of the directly dependent deltas. All of
     * these deltas in the list must be applied before this delta can applied.
     * 
     * @param delta
     *            the delta wich this delta depends on.
     */
    public void addDependeny(IMemoryGraphDelta delta);

    /**
     * Return a collection of all deltas this delta depends. That are all deltas
     * which this delta dependece directly inclusive all deltas the dependent
     * deltas depends on.
     * 
     * @return a collection of all dependend deltas.
     */
    public Collection getAllDependentDeltas();

    /**
     * Check if the delta set contains all necessary dependencies. That means,
     * that the delta set must contains all deltas computed by
     * <code>getAllDependencies</code>.
     * 
     * @param deltaSet
     *            the delta set checked for validity.
     * @return true if the delta set contains all deltas this delta depends on,
     *         otherwise false.
     */
    public boolean containsAllDependencies(DeltaSet deltaSet);

    /**
     * Returns the location, where this delta will be applied.
     * 
     * @return the location, where this delta will be applied.
     */
    public IDeltaLocation getLocation();
    
	/**
	 * Returns true if the execution of this delta depends on the execution of
	 * the given delta, otherwise false.
	 * 
	 * @param delta
	 * @return true if the execution of this delta depends on the execution of
	 *         the given delta, otherwise false.
	 */
	public boolean dependsOn(IMemoryGraphDelta delta);


}
