/*
 * $Id: CreateObjectDeltaLocation.java,v 1.6 2005/11/14 12:58:27 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 01.08.2005 12:37:58 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.delta.location;

import java.util.List;

import org.deltadebugging.ddstate.extractor.MemoryAccessUtil;
import org.deltadebugging.ddstate.extractor.exceptions.MemoryAccessException;
import org.deltadebugging.ddstate.launcher.client.ITestRunner;
import org.deltadebugging.ddstate.launcher.client.TestRunnerException;
import org.deltadebugging.ddstate.memorygraph.IGraphEdge;
import org.deltadebugging.ddstate.memorygraph.delta.create.CreateObjectDelta;
import org.deltadebugging.ddstate.memorygraph.exceptions.DeltaExecutionException;
import org.deltadebugging.ddstate.memorygraph.util.Path;
import org.deltadebugging.ddstate.memorygraph.util.PathComparator;
import org.deltadebugging.ddstate.memorygraph.value.IObjectNode;
import org.deltadebugging.ddstate.memorygraph.value.IStringObjectNode;
import org.deltadebugging.ddstate.memorygraph.value.IValueNode;
import org.eclipse.jdt.debug.core.IJavaThread;
import org.eclipse.jdt.debug.core.IJavaVariable;

/**
 * COMMENTME Comment type CreateObjectDeltaLocation
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class CreateObjectDeltaLocation implements IDeltaLocation {

	private final CreateObjectDelta fCreateObjDelta;

	private final IGraphEdge fEdge;

	public CreateObjectDeltaLocation(CreateObjectDelta createObjDelta,
			IGraphEdge edge) {
		this.fCreateObjDelta = createObjDelta;
		this.fEdge = edge;
	}

	public CreateObjectDeltaLocation(CreateObjectDelta createObjDelta) {
		this.fCreateObjDelta = createObjDelta;
		this.fEdge = null;
	}

	/**
	 * Returns the underlying create delta of this create delta location.
	 * 
	 * @return the underlying create delta of this create delta location.
	 */
	public CreateObjectDelta getCreateOjbectDelta() {
		return this.fCreateObjDelta;
	}

	/**
	 * Returns the graph edge outgoing from the create delta if it was defined,
	 * otherwise <code>null</code>.
	 * 
	 * @return the graph edge outgoing from the create delta if it was defined,
	 *         otherwise <code>null</code>.
	 */
	public IGraphEdge getGraphEdge() {
		return this.fEdge;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.delta.location.IDeltaLocation#getVariableToLocation(org.eclipse.jdt.debug.core.IJavaThread,
	 *      org.deltadebugging.ddstate.launcher.client.ITestRunner)
	 */
	public IJavaVariable getVariableToLocation(IJavaThread thread,
			ITestRunner testRunner) throws DeltaExecutionException {
		IJavaVariable result = null;

		int classLoaderID = -1;
		try {
			classLoaderID = testRunner.getCurrentClassLoaderID();
		} catch (TestRunnerException e) {
			throw new DeltaExecutionException(
					"Cannot retrieve the id of the current class loader "
							+ "of the junit test runner.", e);
		}

		// the delta depends from another delta, we have to compute the path
		// at runtime
		assert this.fCreateObjDelta.getValueToCreate() != null;
		assert this.fCreateObjDelta.getValueToCreate() instanceof IObjectNode;

		IObjectNode createdObj = (IObjectNode) this.fCreateObjDelta
				.getValueToCreate();

		Path path = null;

		if (testRunner.isObjectAlreadyCreated(createdObj)) {
			path = testRunner.getPathToCreatedObject(createdObj);

		} else {
			throw new DeltaExecutionException("required object not created");
		}

		if (this.fEdge != null) {
			List edgeList = path.getAsList();
			edgeList.add(this.fEdge);

			path = new Path(edgeList);
		}

		try {
			result = MemoryAccessUtil.getVariable(thread, path, classLoaderID);
		} catch (MemoryAccessException e1) {
			throw new DeltaExecutionException(
					"Cannot find the variable to the path:\n" + path);
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.delta.location.IDeltaLocation#getLocationAsString()
	 */
	public String getLocationAsString() {
		String result = null;
		IValueNode valueToCreate = this.fCreateObjDelta.getValueToCreate();

		result = "new created " + valueToCreate.getShortReferenceTypeName();

		if (this.fEdge != null) {
			result += "." + this.fEdge.getName();
		}

		if (valueToCreate instanceof IStringObjectNode) {
			result += " (\""
					+ ((IStringObjectNode) valueToCreate)
							.getStringValue() + "\")";
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {

		if (!(obj instanceof CreateObjectDeltaLocation)) {
			return false;
		}

		CreateObjectDeltaLocation creatObjDeltaLocObj = (CreateObjectDeltaLocation) obj;

		Path firstObjPath = this.getCreateOjbectDelta()
				.getLocationToValueToCreate().getPath();
		if (this.getGraphEdge() != null) {
			firstObjPath.addLastGraphEdge(this.getGraphEdge());
		}

		Path secondObjPath = creatObjDeltaLocObj.getCreateOjbectDelta()
				.getLocationToValueToCreate().getPath();
		if (creatObjDeltaLocObj.getGraphEdge() != null) {
			secondObjPath
					.addLastGraphEdge(creatObjDeltaLocObj.getGraphEdge());
		}

		return PathComparator.getInstance()
				.compare(firstObjPath, secondObjPath) == 0;
	}

}
