/*
 * $Id: AbstractGraphNodeValueFilter.java,v 1.3 2005/11/14 10:59:58 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Sep 17, 2004 12:03:05 PM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.filter.nodefilter;

import org.deltadebugging.ddstate.memorygraph.IGraphNode;
import org.deltadebugging.ddstate.memorygraph.value.IValueNode;

/**
 * This abstract class should be extended if you want to create a graph filter
 * only for <code>IValueNode</code>
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public abstract class AbstractGraphNodeValueFilter extends
        AbstractGraphNodeFilter {

    /**
     * Look if the given graph node is a <code>IValueNode</code>. If it
     * is this method return the result of the call of
     * <code>unfoldGraphNodeValue</code>, else it return true.
     * 
     * @param graphNode
     *            the grap node to check.
     * @return true if the given graph node not instance of
     *         <code>IValueNode</code>, else the result of the call
     *         <code>unfoldGraphNodeValue</code> with the given graph node as
     *         argument.
     * @throws NullPointerException
     *             if the given graph node is null.
     */
    public boolean filterGraphNode(IGraphNode graphNode) {
        if (graphNode == null)
            throw new NullPointerException(
                    "The given graph node must not be null");

        boolean result = false;

        if (graphNode instanceof IValueNode) {
            result = this.filterGraphNodeValue((IValueNode) graphNode);
        }

        return result;
    }

    protected abstract boolean filterGraphNodeValue(
            IValueNode graphNodeValue);

}