/*
 * $Id: JavaTypeNameFilter.java,v 1.1 2005/03/24 08:05:32 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 23.03.2005 22:14:43 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.filter.typefilter;

import org.deltadebugging.ddstate.memorygraph.filter.GraphFilterException;
import org.eclipse.debug.core.DebugException;
import org.eclipse.jdt.debug.core.IJavaType;

/**
 * COMMENTME Comment type JavaTypeNameFilter
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class JavaTypeNameFilter extends AbstractJavaTypeFilter {

    private String[] fTypeNames = null;

    public JavaTypeNameFilter(String[] typeNames) {
        this.fTypeNames = typeNames;

        // cut of '*' of package names
        for (int i = 0; i < typeNames.length; i++) {
            String typeName = typeNames[i];
            if (typeName.endsWith("*")) {
                typeNames[i] = typeName.substring(0, typeName.length() - 1);
            }
        }
    }

    public JavaTypeNameFilter(String typeName) {
        this(new String[] { typeName });
    }

    public JavaTypeNameFilter() {
        this(new String[] {});
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.memorygraph.filter.typefilter.IJavaTypeFilter#createJavaTypeNode(org.eclipse.jdt.debug.core.IJavaType)
     */
    public boolean createJavaTypeNode(IJavaType type)
            throws GraphFilterException {
        boolean result = false;

        String typeName = null;
        try {
            typeName = type.getName();
        } catch (DebugException e) {
            throw new GraphFilterException(
                    "Cannot receive the name of the given type", e);
        }

        for (int i = 0; i < this.fTypeNames.length; i++) {
            if (typeName.startsWith(this.fTypeNames[i])) {
                result = true;
            }
        }

        return result;
    }
}