/*
 * $Id: NullValueNode.java,v 1.5 2006/01/03 21:02:54 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 14.09.2004 11:19:03 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.value;

import org.deltadebugging.ddcore.DeltaSet;
import org.deltadebugging.ddstate.comparator.delta.GraphDecorator;
import org.deltadebugging.ddstate.memorygraph.IGraphElement;
import org.deltadebugging.ddstate.memorygraph.IRootNode;
import org.deltadebugging.ddstate.memorygraph.exceptions.DeltaComputationException;
import org.deltadebugging.ddstate.memorygraph.exceptions.MemoryGraphExtractionException;
import org.eclipse.jdt.debug.core.IJavaValue;

/**
 * COMMENTME Comment type NullValueNode
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class NullValueNode extends ValueNode implements INullValueNode {

	// COMMENTME
	public NullValueNode(IRootNode rootNode, IJavaValue value)
			throws MemoryGraphExtractionException {
		super(rootNode, value);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.GraphNode#createSuccessors()
	 */
	protected void createChildren() {
		// null values have not a successor
		return;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.IGraphElement#computeDeltaSet(org.deltadebugging.ddstate.memorygraph.IGraphElement)
	 */
	public DeltaSet computeDeltaSet(IGraphElement element)
			throws DeltaComputationException {

		assert element instanceof INullValueNode;
		
		if (!this.isMaskedAs(GraphDecorator.NOT_COMPARED)) {
			return new DeltaSet();
		}
		
		GraphDecorator.decorateGraphElementsAsEqual(this, element);

		return new DeltaSet();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.IGraphElement#graphEquals(org.deltadebugging.ddstate.memorygraph.IGraphElement)
	 */
	public boolean graphEquals(IGraphElement element) {
		if (!(element instanceof INullValueNode)) {
			return false;
		}

		return true;
	}

}