/*
 * $Id: ObjectFieldEdge.java,v 1.3 2005/11/28 15:28:44 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Aug 26, 2005 1:38:32 PM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.value;

import org.deltadebugging.ddstate.memorygraph.GraphEdge;
import org.eclipse.jdt.core.Signature;

/**
 * This edge joins a field node (IValueNode) with the class object reference
 * (IClassObjectReferenceNode) that contains the field.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann</a>
 */
public class ObjectFieldEdge extends GraphEdge implements IObjectFieldEdge {

    private final String fFieldName;

    private final String fDeclTypeSignature;

    private final boolean fSuperclassField;

    public ObjectFieldEdge(IObjectNode sourceNode, IValueNode targetNode,
            String fieldName, String declaringTypeSignature,
            boolean superclassField) {
        super(sourceNode, targetNode, null);

        this.fFieldName = fieldName;
        this.fDeclTypeSignature = declaringTypeSignature;
        this.fSuperclassField = superclassField;

        if (!superclassField) {
            this.fName = fieldName;
        } else {
            this.fName = "(" + this.getDeclaringTypeName() + ")" + fieldName;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.memorygraph.value.IObjectFieldEdge#getDeclaringTypeName()
     */
    public String getDeclaringTypeName() {
        String result = Signature
                .getSignatureSimpleName(this.fDeclTypeSignature);
        result = result.replace('/', '.');

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.memorygraph.value.IObjectFieldEdge#getDeclaringTypeSignature()
     */
    public String getDeclaringTypeSignature() {
        return this.fDeclTypeSignature;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.memorygraph.value.IObjectFieldEdge#getFieldName()
     */
    public String getFieldName() {
        return this.fFieldName;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.memorygraph.value.IObjectFieldEdge#isSuperclassField()
     */
    public boolean isSuperclassField() {
        return this.fSuperclassField;
    }

}
