/*
 * $Id: StringObjectNode.java,v 1.4 2006/01/03 21:02:54 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 30.08.2004 15:56:07 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.value;

import org.deltadebugging.ddcore.DeltaSet;
import org.deltadebugging.ddstate.comparator.delta.GraphDecorator;
import org.deltadebugging.ddstate.memorygraph.IGraphElement;
import org.deltadebugging.ddstate.memorygraph.IRootNode;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdateStringDelta;
import org.deltadebugging.ddstate.memorygraph.exceptions.DeltaComputationException;
import org.deltadebugging.ddstate.memorygraph.exceptions.MemoryGraphExtractionException;
import org.eclipse.debug.core.DebugException;
import org.eclipse.jdt.debug.core.IJavaObject;

/**
 * COMMENTME Comment type ClassObjectReferenceNode
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class StringObjectNode extends ObjectNode implements IStringObjectNode {

	private String string = null;

	// COMMENTME
	public StringObjectNode(IRootNode rootNode, IJavaObject value)
			throws MemoryGraphExtractionException {
		super(rootNode, value);

		try {
			this.string = value.getValueString();
		} catch (DebugException e) {
			throw new MemoryGraphExtractionException(
					"Cannot retrieve the value string of the given variable", e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.GraphNode#createChildren()
	 */
	protected void createChildren() throws MemoryGraphExtractionException {
		// Strings have no successors
		return;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.value.IStringObjectNode#getString()
	 */
	public String getStringValue() {
		return this.string;
	}

	public DeltaSet computeDeltaSet(IGraphElement element)
			throws DeltaComputationException {

		if (!this.isMaskedAs(GraphDecorator.NOT_COMPARED)) {
			return new DeltaSet();
		}

		DeltaSet result = new DeltaSet();

		assert element instanceof IStringObjectNode;
		IStringObjectNode stringRef = (IStringObjectNode) element;
		assert this.getReferenceTypeName().equals(
				stringRef.getReferenceTypeName());

		if (!this.getStringValue().equals(stringRef.getStringValue())) {
			// mask as changed
			GraphDecorator.decorateGraphElementsAsDifferent(this, stringRef);

			result.add(new UpdateStringDelta(this, stringRef));
		} else {

			// mask as equal
			GraphDecorator.decorateGraphElementsAsEqual(this, element);
		}

		return result;
	}

	public boolean graphEquals(IGraphElement element) {
		if (!(element instanceof IStringObjectNode)) {
			return false;
		}

		IStringObjectNode objRef = ((IStringObjectNode) element);

		boolean result = super.graphEquals(element)
				&& this.getStringValue().equals(objRef.getStringValue());

		return result;
	}
}