/*
 * $Id: CreateStepFilterDialog.java,v 1.2 2005/08/09 15:52:02 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University Prof. Dr. Andreas Zeller
 * 
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Common Public License v1.0 which accompanies
 * this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Created on 03.08.2005 10:15:03 by @author Karsten Lehmann Mail to:
 * lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.ui.preferences;

import org.deltadebugging.ddstate.DDStatePlugin;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jdt.debug.ui.IJavaDebugUIConstants;
import org.eclipse.jdt.internal.debug.ui.DialogSettingsHelper;
import org.eclipse.jdt.internal.debug.ui.Filter;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class CreateStepFilterDialog extends StatusDialog {

    private static final String DEFAULT_NEW_FILTER_TEXT = ""; //$NON-NLS-1$

    Text text;
    private Filter filter;
    private Button okButton;

    boolean filterValid;
    private boolean okClicked;
    private Filter[] existingFilters;

    private CreateStepFilterDialog(Shell parent, Filter filter,
            Filter[] existingFilters) {
        super(parent);
        setShellStyle(getShellStyle() | SWT.RESIZE);
        this.filter = filter;
        this.existingFilters = existingFilters;

        setTitle("Add Step Filter"); //$NON-NLS-1$
        setStatusLineAboveButtons(false);

    }

    static Filter showCreateStepFilterDialog(Shell parent,
            Filter[] existingFilters) {
        CreateStepFilterDialog createStepFilterDialog = new CreateStepFilterDialog(
                parent, new Filter(DEFAULT_NEW_FILTER_TEXT, true),
                existingFilters);
        createStepFilterDialog.create();
        createStepFilterDialog.open();

        return createStepFilterDialog.filter;
    }

    protected void createButtonsForButtonBar(Composite parent) {
        this.okButton = createButton(parent, IDialogConstants.OK_ID,
                IDialogConstants.OK_LABEL, true);
        this.okButton.setEnabled(false);
        createButton(parent, IDialogConstants.CANCEL_ID,
                IDialogConstants.CANCEL_LABEL, false);
    }

    protected Control createDialogArea(Composite parent) {
        Composite container = (Composite) super.createDialogArea(parent);

        GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns = 2;
        gridLayout.marginHeight = 15;
        gridLayout.marginWidth = 15;
        container.setLayout(gridLayout);

        int textStyles = SWT.SINGLE | SWT.LEFT;
        Label label = new Label(container, textStyles);
        label.setText("&Pattern to filter:"); //$NON-NLS-1$
        label.setFont(container.getFont());

        // create & configure Text widget for editor
        // Fix for bug 1766. Border behavior on for text fields varies per
        // platform.
        // On Motif, you always get a border, on other platforms,
        // you don't. Specifying a border on Motif results in the characters
        // getting pushed down so that only there very tops are visible. Thus,
        // we have to specify different style constants for the different
        // platforms.
        if (!SWT.getPlatform().equals("motif")) { //$NON-NLS-1$
            textStyles |= SWT.BORDER;
        }

        this.text = new Text(container, textStyles);
        GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
        gridData.horizontalSpan = 1;
        gridData.widthHint = 300;
        this.text.setLayoutData(gridData);
        this.text.setFont(container.getFont());

        this.text.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                validateChange();
                if (!CreateStepFilterDialog.this.filterValid)
                    updateStatus(new Status(
                            IStatus.ERROR,
                            DDStatePlugin.PLUGIN_ID,
                            IStatus.ERROR,
                            "Step Filter must be a valid Java identifier.", null)); //$NON-NLS-1$
                else if (isDuplicateFilter(CreateStepFilterDialog.this.text
                        .getText().trim())) {
                    updateStatus(new Status(IStatus.WARNING,
                            DDStatePlugin.PLUGIN_ID, IStatus.WARNING,
                            "Step Filter already exists.", null)); //$NON-NLS-1$
                    return;
                } else
                    updateStatus(Status.OK_STATUS);
            }
        });

        return container;
    }
    
    public void updateStatus(IStatus status) {
        super.updateStatus(status);
    }

    void validateChange() {
        String trimmedValue = this.text.getText().trim();

        if (trimmedValue.length() > 0 && validateInput(trimmedValue)) {
            this.okButton.setEnabled(true);
            this.filter.setName(this.text.getText());
            this.filterValid = true;
        } else {
            this.okButton.setEnabled(false);
            this.filter.setName(DEFAULT_NEW_FILTER_TEXT);
            this.filterValid = false;
        }
    }

    boolean isDuplicateFilter(String trimmedValue) {
        for (int i = 0; i < this.existingFilters.length; i++)
            if (this.existingFilters[i].getName().equals(trimmedValue))
                return true;
        return false;
    }

    /**
     * A valid step filter is simply one that is a valid Java identifier. and,
     * as defined in the JDI spec, the regular expressions used for step
     * filtering must be limited to exact matches or patterns that begin with
     * '*' or end with '*'. Beyond this, a string cannot be validated as
     * corresponding to an existing type or package (and this is probably not
     * even desirable).
     */
    private boolean validateInput(String trimmedValue) {
        char firstChar = trimmedValue.charAt(0);
        if (!Character.isJavaIdentifierStart(firstChar)) {
            if (!(firstChar == '*')) {
                return false;
            }
        }
        int length = trimmedValue.length();
        for (int i = 1; i < length; i++) {
            char c = trimmedValue.charAt(i);
            if (!Character.isJavaIdentifierPart(c)) {
                if (c == '.' && i != (length - 1)) {
                    continue;
                }
                if (c == '*' && i == (length - 1)) {
                    continue;
                }
                return false;
            }
        }
        return true;
    }

    /**
     * Returns the name of the section that this dialog stores its settings in
     * 
     * @return String
     */
    protected String getDialogSettingsSectionName() {
        return IJavaDebugUIConstants.PLUGIN_ID
                + ".CREATE_STEP_FILTER_DIALOG_SECTION"; //$NON-NLS-1$
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.window.Window#getInitialLocation(org.eclipse.swt.graphics.Point)
     */
    protected Point getInitialLocation(Point initialSize) {
        Point initialLocation = DialogSettingsHelper
                .getInitialLocation(getDialogSettingsSectionName());
        if (initialLocation != null) {
            return initialLocation;
        }
        return super.getInitialLocation(initialSize);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.window.Window#getInitialSize()
     */
    protected Point getInitialSize() {
        Point size = super.getInitialSize();
        return DialogSettingsHelper.getInitialSize(
                getDialogSettingsSectionName(), size);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.window.Window#close()
     */
    public boolean close() {
        if (!this.okClicked) {
            this.filterValid = false;
            this.filter = null;
        }
        DialogSettingsHelper.persistShellGeometry(getShell(),
                getDialogSettingsSectionName());
        return super.close();
    }

    protected void okPressed() {
        this.okClicked = true;
        super.okPressed();
    }
}